/*
 * A custom control that draws a simple spark line graph
 * using SVG, which is very lightweight and fast.
 */
sap.ui.define([
	"sap/ui/core/Control"
], function (Control) {
	"use strict";
	return Control.extend("sas.fscf.control.Sparkline", {
		metadata : {
			properties: {
                width           : {type : "int", defaultValue : 36},
                height         	: {type : "int", defaultValue : 20},
                padding			: {type : "string"},
                fillColor		: {type : "string", defaultValue : "#c0d0f0"},
                lineColor		: {type : "string", defaultValue : "#0000f0"},
                dotColor		: {type : "string", defaultValue : "#f08000"},
                yValues			: {type : "float[]"},
                xLabels			: {type : "string[]"}
			}
		},
		init : function () {
		},
		renderer : function (oRm, oControl) {
			oRm.write("<div");
			oRm.writeControlData(oControl);
//			oRm.addClass("cfSparkline");
//			oRm.writeClasses();
			oRm.write(">");

			var yValues = oControl.getYValues();
//			yValues = [5,1,18,18,5,1,1,18,0,12];	// temp override			
			
			var w = oControl.getWidth();
			var h = oControl.getHeight();
			if (yValues != undefined && yValues.length > 1) {
				var line_w = 2;
				var r = 2;
				var base_x = r + 1;
				// leave a base_x size gutter around the edge so the dots don't get clipped; 
				// half of the dots at the edge coordinates will draw in the gutter
				var draw_w = w - (base_x * 2);
				var draw_h = h - (base_x * 2);
				var min = Math.min.apply(null, yValues);
				if (min < 0) {
					// normalize so all numbers are positive
					for (var i = 0; i < yValues.length; i++) {
						yValues[i] = yValues[i] + Math.abs(min);
					}
				}
				var max = Math.max.apply(null, yValues);
				min = Math.min.apply(null, yValues);
				var scale_y = (max == 0) ? 1 : draw_h / max;
				var delta_x = draw_w / Math.max(yValues.length - 1, 1);
				var base_h = h - base_x;
				var x = base_x;
								
				oRm.write('<svg width="' + w + '" height="' + h + '"');
				if (oControl.getPadding() != undefined) {
					oRm.write(' style="padding:' + oControl.getPadding() + '"');
				}
				oRm.write('>');
				oRm.write('<g>');
				// polyfill
				oRm.write('<g stroke-width="1px" stroke="' + oControl.getFillColor() + '" fill="' + oControl.getFillColor() + '" vector-effect="non-scaling-stroke">');
				for (var i = 0; i < yValues.length - 1; i++) {
					if (i == 0) {
						oRm.write('<path d="M ' + x + ' ' + (base_h - (yValues[i] * scale_y)) + ' L ' + (x + delta_x) + ' ' + (base_h - (yValues[i + 1] * scale_y)));
					} else {
						oRm.write(' L ' + (x + delta_x) + ' ' + (base_h - (yValues[i + 1] * scale_y)));
					}
					if (i == yValues.length - 2) {
						oRm.write(' L ' + (x + delta_x) + ' ' + base_h + ' L ' + base_x + ' ' + base_h + ' Z" />');
					}
					x += delta_x;
				}
				oRm.write('</g>');
				// polyline
				oRm.write('<g stroke-width="' + line_w + 'px" stroke="' + oControl.getLineColor() + '" fill="none" vector-effect="non-scaling-stroke">');
				x = base_x;
				for (var i = 0; i < yValues.length - 1; i++) {
					if (i == 0) {
						oRm.write('<polyline points="' + x + ',' + (base_h - (yValues[i] * scale_y)) + ' ' + (x + delta_x) + ',' + (base_h - (yValues[i + 1] * scale_y)));
					} else {
						oRm.write(' ' + (x + delta_x) + ',' + (base_h - (yValues[i + 1] * scale_y)));
					}
					if (i == yValues.length - 2) {
						oRm.write('" />');
					}
					x += delta_x;
				}
				oRm.write('</g>');
				// dots
				oRm.write('<g stroke-width="1px" stroke="' + oControl.getDotColor() + '" fill="' + oControl.getDotColor() + '">');
				x = base_x;
				var bMinDrawn, bMaxDrawn = false;
				for (var i = 0; i < yValues.length; i++) {
					// only draw dots for the first, last, first min and first max values
					if (i == 0 || i == yValues.length - 1 || (yValues[i] == min && !bMinDrawn) || (yValues[i] == max && !bMaxDrawn)) {
						oRm.write('<circle cx="' + x + '" cy="' + (base_h - (yValues[i] * scale_y)) + '" r="' + r + '" />');
						if (yValues[i] == min) {
							bMinDrawn = true;
						}
						if (yValues[i] == max) {
							bMaxDrawn = true;
						}
					}
					x += delta_x;
				}
				oRm.write('</g>');
				oRm.write('</g>');
				oRm.write('</svg>');
			} else {
				// S1554553
				// put an empty placeholder svg tag of the proper size
				// if the sparkline is drawn in the cell of a table, for example, this ensures that each table row height will be consistent
				oRm.write('<svg width="' + w + '" height="' + h + '"');
				if (oControl.getPadding() != undefined) {
					oRm.write(' style="padding:' + oControl.getPadding() + '"');
				}
				oRm.write('>');
				oRm.write('</svg>');
			}
			oRm.write("</div>");
		}
	});
});