jQuery.sap.require("sas.fscf.views.PrimaryTableView");

sap.ui.jsview("sas.fscf.views.RelatedRows", {

	getControllerName: function() {
    	// mix in PrimaryTableView
    	jQuery.extend(true, this, sas.fscf.views.PrimaryTableView);

		return "sas.fscf.views.RelatedRows";
	},

	_alertTypes: [],
	_analystListsFieldTypes: [],

    getFieldtypesUrl: function() {
		var fieldtypesUrl = "rest/";
		if (this.isExplore()) {
			fieldtypesUrl += "businessunits/fieldtypes";
		} else {
			fieldtypesUrl += "businessunits/" + this.getBUId() + "/fieldtypes?counts=true";
			if (!this.isEstimation() &&
				sas.fscf.getCurrentPageView().jsonRule.transactionTypeId &&
	       		sas.fscf.getCurrentPageView().jsonRule.transactionTypeId != "-1") {
	   			fieldtypesUrl += "&txnType=" + sas.fscf.getCurrentPageView().jsonRule.transactionTypeId;
			}
			if (this.isCoding()) {
				// for coding approach, only read-only variables can be manually added and removed as columns in the table.
				// read-write variables (such as user variables, output variables and RUR variables must be automatically
				// added and removed as columns in the table by us if and only if they are referenced in the rule code.
				// SAS will throw an exception during rule test if we ask for any read-write variables that were not referenced 
				// in the rule code, so we have to dynamically/automatically add and remove the read-write variables as 
				// columns for the user.
	   			fieldtypesUrl += "&filter=access==\'read-only\'";
			}
		}
		return fieldtypesUrl;
    },

	getTableIdSuffix: function() {
		return "_TransactionTable";
	},

	attachContextMenu: function(table) {
		var _this = this;
		if (this.isExplore()) {
			this._bCanCreateAlert = sas.fscf.userHasPrivilege("MANALERT");
			this._bCanAddToAnalystList = sas.fscf.userHasPrivilege("ADDDATLIST");
			this._bCanRemoveFromAnalystList = sas.fscf.userHasPrivilege("RMDATLIST");

	    	sas.ajax({url:'rest/alerttypes', headers:sas.fscf.getAjaxHeaders()})
			  .done(function(oData){
				  _this._alertTypes = oData.items;
			}).fail(function(oObj,textStatus){
		         console.info("Failed to get alert types.");
			});

	    	sas.ajax({url:'rest/analystlists/fieldtypes', headers:sas.fscf.getAjaxHeaders()})
			  .done(function(oData){
				  _this._analystListsFieldTypes = oData.items;
			}).fail(function(oObj,textStatus){
		         console.info("Failed to get alert types.");
			});

			var oMenu = new sas.hc.ui.unified.Menu(table.sId + "_Menu");
			// set name attribute
	    	oMenu.onAfterRendering = function() {
	        	if (sas.hc.ui.unified.Menu.prototype.onAfterRendering) {
	        		sas.hc.ui.unified.Menu.prototype.onAfterRendering.apply(this);
	        	}
	        	// must be set after rendering when the dom ref will exist
	        	this.getDomRef().setAttribute("name", "Menu");
	    	};
	        table.setAggregation("contextMenu", oMenu);
	        table.attachCellContextmenu(function(oEvent) {
				_this._contextField = oEvent.oSource.getColumns()[parseInt(oEvent.mParameters.columnIndex)].getSortProperty();
				_this._contextAlertType = _this.getAlertTypeForField(_this._contextField);

				// table could be scrolled down, so factor in first visible row
				var row = oEvent.oSource.getRows()[oEvent.mParameters.rowIndex - oEvent.oSource.getFirstVisibleRow()];
				if (row) {
					_this._contextValue = row.getCells()[parseInt(oEvent.mParameters.columnIndex)].getText();
				} else {
					_this._contextValue = "";
				}
				if (row) {
					// table could be sorted, so factor in index from binding
					var rowData = row.getModel().getData().items[table.getBinding("rows").aIndices[row.getIndex()]];
					if (rowData) {
						_this._contextTxnId = "" + rowData["CMX_TRAN_ID"];
						_this._contextTxnAcronym = rowData["SMH_ACCT_TYPE"] + rowData["SMH_ACTIVITY_TYPE"];
					} else {
						_this._contextTxnId = "";
						_this._contextTxnAcronym = "";
					}
				} else {
					_this._contextTxnId = "";
					_this._contextTxnAcronym = "";
				}
				var bEnabled = table.getModel().getData() && (table.getModel().getData().count > 0) && (_this._contextTxnId.length > 0);
       		    oMenuItem.setEnabled(_this._bCanCreateAlert && bEnabled);
       		    oMenuItemAddToList.setEnabled(_this._bCanAddToAnalystList && bEnabled);
       		    oMenuItemRemoveFromList.setEnabled(_this._bCanRemoveFromAnalystList && bEnabled);
       		});

	        var oMenuItem = new sas.hc.ui.unified.MenuItem({
	        	id: table.sId + "_CreateAlertMenuItem",
	            text: sas.fscf.rb.getText("rules.rulesPage.guidedView.form.actions.CreateAnAlert.txt"),
	            enabled: false
	        });
	        oMenuItem.attachSelect(function(oEvent) {
	        	_this.getActionConfiguration().open();
	        });
			// set name attribute
	    	oMenuItem.onAfterRendering = function() {
	        	if (sas.hc.ui.unified.MenuItem.prototype.onAfterRendering) {
	        		sas.hc.ui.unified.MenuItem.prototype.onAfterRendering.apply(this);
	        	}
	        	// must be set after rendering when the dom ref will exist
	        	this.getDomRef().setAttribute("name", "CreateAlertMenuItem");
	    	};
	        oMenu.addItem(oMenuItem);

	        var oMenuItemAddToList = new sas.hc.ui.unified.MenuItem({
	        	id: table.sId + "_AddToListMenuItem",
	            text: sas.fscf.rb.getText("primaryTable.relatedRows.actions.AddToList.txt"),
	            enabled: false,
                startsSection: true
	        });
	        oMenuItemAddToList.attachSelect(function(oEvent) {
	        	_this.getAnalystListDialog(true).open();
	        });
			// set name attribute
	        oMenuItemAddToList.onAfterRendering = function() {
	        	if (sas.hc.ui.unified.MenuItem.prototype.onAfterRendering) {
	        		sas.hc.ui.unified.MenuItem.prototype.onAfterRendering.apply(this);
	        	}
	        	// must be set after rendering when the dom ref will exist
	        	this.getDomRef().setAttribute("name", "AddToListMenuItem");
	    	};
	        oMenu.addItem(oMenuItemAddToList);

	        var oMenuItemRemoveFromList = new sas.hc.ui.unified.MenuItem({
	        	id: table.sId + "_RemoveFromListMenuItem",
	            text: sas.fscf.rb.getText("primaryTable.relatedRows.actions.RemoveFromList.txt"),
	            enabled: false
	        });
	        oMenuItemRemoveFromList.attachSelect(function(oEvent) {
	        	_this.getAnalystListDialog(false).open();
	        });
			// set name attribute
	        oMenuItemRemoveFromList.onAfterRendering = function() {
	        	if (sas.hc.ui.unified.MenuItem.prototype.onAfterRendering) {
	        		sas.hc.ui.unified.MenuItem.prototype.onAfterRendering.apply(this);
	        	}
	        	// must be set after rendering when the dom ref will exist
	        	this.getDomRef().setAttribute("name", "RemoveFromListMenuItem");
	    	};
	        oMenu.addItem(oMenuItemRemoveFromList);
		}
	},

	getDefaultColumns: function() {
		var defaultColumns;
		if (this.isExplore()) {
			defaultColumns = sas.config.get('fscf').exploreDefaultRelatedRowsColumns;
          	var cf_lastExploreRelatedRowsColumns = sas.fscf.getCookie("cf_lastExploreRelatedRowsColumns");
          	if (cf_lastExploreRelatedRowsColumns) {
          		defaultColumns = cf_lastExploreRelatedRowsColumns;
          	}
		} else if (this.isEstimation()) {
   			defaultColumns = sas.config.get('fscf').exploreDefaultRelatedRowsColumns;
           	var estimation = this.getEstimation();
           	if (estimation) {
           		var ruleId;
           		if (estimation.allRuleIds) {
               		// If there are all rule ids, then we're looking at a all rules and get and use the columns from the first rule.
           			ruleId = estimation.allRuleIds[0]; 
           		} else if (estimation.ruleId) {
               		// If there is a rule id, then we're looking at a single rule and get and use the columns from the rule.
           			ruleId = estimation.ruleId;
           		}
           		if (ruleId) {
        			// this call is synchronous
        			var jqXHR = sas.ajax({url:"rest/businessunits/" + this.getBUId() + "/rules?filter=id==" + ruleId + "&fields=expressions", type: "GET", async: false, headers:sas.fscf.getAjaxHeaders()});
        			
        	  	  	if (jqXHR.status == 200) {
        	  	  	  var oData = jqXHR.responseJSON;
        	  		  if (oData.items && oData.items[0] && oData.items[0].expressions) {
        	  			  var expressions = oData.items[0].expressions;
        	  			  if (expressions.expressions[0].properties &&
        	  				  expressions.expressions[0].properties.defaultRelatedRowsColumns &&
        	  				  expressions.expressions[0].properties.defaultRelatedRowsColumns.length > 0) {
        	  				    defaultColumns = expressions.expressions[0].properties.defaultRelatedRowsColumns;
        	  			  }
        	  		  }
        	  	  	} else {
        	  	  	  console.info("an error occurred getting the rule expressions");
        	  	  	}           		
           		}
           	}           	
		} else {
			defaultColumns = sas.config.get('fscf').defaultRelatedRowsColumns;
			if (sas.fscf.getCurrentPageView().jsonRule.expressions.expressions[0].properties &&
				sas.fscf.getCurrentPageView().jsonRule.expressions.expressions[0].properties.defaultRelatedRowsColumns &&
				sas.fscf.getCurrentPageView().jsonRule.expressions.expressions[0].properties.defaultRelatedRowsColumns.length > 0) {
				defaultColumns = sas.fscf.getCurrentPageView().jsonRule.expressions.expressions[0].properties.defaultRelatedRowsColumns;
			}
		}
		return defaultColumns;
	},
	
	pruneFieldsByAccess: function(fields) {
		if (this.isCoding()) {
			// if it's coding approach, only use the fields that are read-only. read-write fields must be
			// handled dynamically/automatically through contextSpecificFields (only if referenced in the rule code)
	    	for (var i = fields.length - 1; i >= 0; i--) {
	    		if (this.getFieldAccess(fields[i]) != "read-only") {
					fields.splice(i, 1); // remove the field
	    		}
	    	}
		}
	},

    padFields: function(fields) {
    	if (this.isExplore()) {
    		// make sure CMX_TRAN_ID, SMH_ACCT_TYPE & SMH_ACTIVITY_TYPE are always in fields,
    		// as their values are needed to determine _contextTxnId & _contextTxnAcronym for Create an Alert
    		var newArray = fields.slice(0); // clone the array
    		if ($.inArray("CMX_TRAN_ID", newArray) === -1) {
    			newArray[newArray.length] = "CMX_TRAN_ID";
    		}
    		if ($.inArray("SMH_ACCT_TYPE", newArray) === -1) {
    			newArray[newArray.length] = "SMH_ACCT_TYPE";
    		}
    		if ($.inArray("SMH_ACTIVITY_TYPE", newArray) === -1) {
    			newArray[newArray.length] = "SMH_ACTIVITY_TYPE";
    		}
        	return newArray;
    	}
    	return fields;
    },

    storeFields: function() {
    	var fields = this._fields;
    	if (this.isCoding()) {
    		// If it's coding approach, don't store any of the context specific fields
    		fields = [];
    		for (var i = 0; i < this._fields.length; i++) {
    			if ($.inArray(this._fields[i], this._contextSpecificFields) == -1) {
        			fields[fields.length] = this._fields[i];
    			}
    		}
    	}
    	var fieldsStr = fields.join(" ");
    	if (this.isExplore()) {
    		sas.fscf.setCookie("cf_lastExploreRelatedRowsColumns", fieldsStr, sas.fscf.getNYearsFromNow(1));
		} else if (this.isEstimation()) {
       		// If it's an estimation, then any column customizations in the table are temporary and not stored
        	return;
    	} else {
    		if (!sas.fscf.getCurrentPageView().jsonRule.expressions.expressions[0].properties) {
    			sas.fscf.getCurrentPageView().jsonRule.expressions.expressions[0].properties = {};
    		}
    		sas.fscf.getCurrentPageView().jsonRule.expressions.expressions[0].properties.defaultRelatedRowsColumns = fieldsStr;
    		sas.fscf.getCurrentPageView().setDirty(true);
    	}
    },

    getSchema: function() {
		var schema = null;
		if (this.isExplore()) {
			schema = "esdb";
		} else if (this.isCoding()) {
			schema = "swsrdb";
		} else if (this.isEstimation()) {
			schema = "swsedb";
		}
		return schema;
    },

    getEstimationFilter: function (estimation) {
        let answer = " and (estimation.id==" + estimation.estimationId;
        if (estimation.ruleId) {
            answer += " and id==" + estimation.ruleId;
            answer += " and rule.businessUnitId==" + this.getBUId();
        }
        if (estimation.forAlerts) {
            answer += " and ruleTypeName=='Queue'";
        }
        return answer += ")";
    },

    getContextSpecificFields: function() {
    	var contextSpecificFields = [];
		if (this.isCoding() || this.isEstimation()) {
			var filter = "(";
			if (this.isEstimation()) {
				if (this.getEstimation().allRuleIds) {
					if (this.getEstimation().allRuleIds.length > 0) {
						filter += "id==" + this.getEstimation().allRuleIds[0];
					}
					if (this.getEstimation().allRuleIds.length > 1) {
						for (var j = 1; j < this.getEstimation().allRuleIds.length; j++) {
							filter += " or id==" + this.getEstimation().allRuleIds[j];
						}
					}
				} else if (this.getEstimation().ruleId) {
					filter += "id==" + this.getEstimation().ruleId; 
				}
			} else {
				filter += "id==" + sas.fscf.getCurrentPageView().jsonRule.id; 
			}
			filter += ")";
			// this call is synchronous
			var jqXHR = sas.ajax({url:"rest/businessunits/" + this.getBUId() + "/rules?filter=" + filter + "&fields=ruleVariables", type: "GET", async: false, headers:sas.fscf.getAjaxHeaders()});
				
	  	  	if (jqXHR.status == 200) {
	  	  	  var oData = jqXHR.responseJSON;
	  		  if (oData.items) {
	  			 for (var k = 0; k < oData.items.length; k++) {
	  				 if (oData.items[k] && oData.items[k].ruleVariables) {
						 for (var i = 0; i < oData.items[k].ruleVariables.length; i++) {
							 if (oData.items[k].ruleVariables[i].altName) {
								 if ($.inArray(oData.items[k].ruleVariables[i].altName, contextSpecificFields) == -1) {
									 contextSpecificFields[contextSpecificFields.length] = oData.items[k].ruleVariables[i].altName; 
								 }
							 }
						 }
	  				 }	  				 
	  			 } 
	  		  }
	  	  	} else {
	  	  	  console.info("an error occurred getting the context specific fields");
	  	  	}
		}
		return contextSpecificFields;
    },
    
    getQueryUrl: function() {
    	if (this.isCoding() || this.isEstimation()) {
    		var url = 'rest/transactions/search?filter=businessUnitId==' + this.getBUId();
    		if (this.isCoding()) {
    			url += ' and id==' + sas.fscf.getCurrentPageView().jsonRule.id;
    		} else if (this.isEstimation()) {
                url += this.getEstimationFilter(this.getEstimation());
    		}
    		url += '&schema=' + this.getSchema() + '&fields=' + encodeURIComponent(this.padFields(this._fields).join(","));
    		return url;
    	}
    	return sas.fscf.getQueryUrlFromRuleModel("search", this.getSchema(), this.padFields(this._fields))
    },

	annotateHeaderLabel: function(oLabel, fieldName) {
		if (this.getAlertTypeForField(fieldName) != null) {
			oLabel._alertable = true;
		}
		oLabel._inAnalystList = this.isInAnalystList(fieldName);
		oLabel.onAfterRendering = function() {
           	if (sas.hc.m.Label.prototype.onAfterRendering) {
           		sas.hc.m.Label.prototype.onAfterRendering.apply(this);
           	}

           	if (this._inAnalystList) {
    			$label = $("#" + this.sId);
		    	$label.find('.cfHeaderInAnalystList').remove(); // remove any icon that might already be there
    			$("<span class=\"sapUiBtnIco sapUiBtnIcoL sapUiIcon cfHeaderInAnalystList\" style=\"font-family: sas-icons;\" data-sap-ui-icon-content=\"&#xf0bb;\" title=\"" + sas.fscf.rb.getText("primaryTable.relatedRows.inList.txt") + "\"></span>").insertAfter($label.find(".sasMLabelText"));
           	}
           	if (this._alertable) {
    			$label = $("#" + this.sId);
		    	$label.find('.cfHeaderAlertable').remove(); // remove any icon that might already be there
    			$("<span class=\"sapUiBtnIco sapUiBtnIcoL sapUiIcon cfHeaderAlertable\" style=\"font-family: sas-icons;\" data-sap-ui-icon-content=\"&#xf018;\" title=\"" + sas.fscf.rb.getText("primaryTable.relatedRows.alertable.txt") + "\"></span>").insertAfter($label.find(".sasMLabelText"));
           	}
    	};
	},

	getAlertTypeForField: function(fieldName) {
		for (var i = 0; i < this._alertTypes.length; i++) {
			if (this._alertTypes[i].AlertableEntityField.toLowerCase() == fieldName.toLowerCase()) {
				return this._alertTypes[i];
			}
		}
		return null;
	},

	isInAnalystList: function(fieldName) {
		for (var i = 0; i < this._analystListsFieldTypes.length; i++) {
			if (this._analystListsFieldTypes[i].name.toLowerCase() == fieldName.toLowerCase()) {
				return true;
			}
		}
		return false;
	},

    getActionConfiguration: function(){
    	var view = sap.ui.jsfragment({
            type: sap.ui.core.mvc.ViewType.JS,
            id: this.sId + "AlertCreationConfigurationDialog",
            fragmentName: "sas.fscf.views.fragments.AlertCreationConfigurationDialog"
        });
    	view.setExplore(true);
    	view.setExploreContext({"txnId":this._contextTxnId, "txnAcronym":this._contextTxnAcronym, "fieldName":this._contextField, "fieldValue":this._contextValue, "alertType":this._contextAlertType});
    	return view.lazyLoadContent();
    },

    getAnalystListDialog: function(bAdd){
    	var view = sap.ui.jsfragment({
            type: sap.ui.core.mvc.ViewType.JS,
            id: this.sId + "AnalystListDialog",
            fragmentName: "sas.fscf.views.fragments.AnalystListDialog"
        });
    	view.setAdd(bAdd);
    	view.setExplore(true);
    	view.setExploreContext({"txnId":this._contextTxnId, "txnAcronym":this._contextTxnAcronym, "fieldName":this._contextField, "fieldValue":this._contextValue, "alertType":this._contextAlertType});
    	return view.lazyLoadContent();
    }
});