jQuery.sap.require("sas.hc.m.MessageDialog");
jQuery.sap.require("sas.hc.m.MessageToast");
jQuery.sap.require("sas.fscf.views.PrimaryView");
jQuery.sap.require("sas.fscf.views.estimation.EstimationHelper");

const EstimationHelper = sas.fscf.views.estimation.EstimationHelper;
const SelectionMode = EstimationHelper.SelectionMode;
const ButtonEnabler = EstimationHelper.ButtonEnabler;
const getText = EstimationHelper.getText;

sap.ui.jsview("sas.fscf.views.RulesPageEstimations", {
	Fast: getText("rules.rulesPage.estimations.fast.txt"),
	Slow: getText("rules.rulesPage.estimations.slow.txt"),

	
    getControllerName: function () {
    	// mix in PrimaryView
    	jQuery.extend(true, this, sas.fscf.views.PrimaryView);
    	return "sas.fscf.views.RulesPageEstimations";
    },

    _columns: {
    	"Main": [
	    	{property: "id",
	    		label: getText("rules.rulesPage.rules.table.column.id.txt"),
	    		width: 60,
	    		formatter: 0,
	    	},
	    	{property: "name",
	    		label: getText("rules.rulesPage.rules.table.column.name.txt"),
	    		width: 180,
	    		formatter: 0,
	    	},
	    	{property: "status",
	    		label: getText("rules.rulesPage.estimations.table.column.status.txt"),
	    		width: 100,
	    		formatter: "Status",
	    	},
	    	{property: "transactionCount",
	    		label: getText("rules.rulesPage.estimations.table.column.read.txt"),
	    		width: 60,
	    		formatter: "Count",
	    	},
	    	{property: "transactionFiredCount",
	    		label: getText("rules.rulesPage.estimations.table.column.fired.txt"),
	    		width: 60,
	    		formatter: "Count",
	    	},
	    	{property: "jobStartTime",
	    		label: getText("rules.rulesPage.estimations.table.column.startTime.txt"),
	    		width: 160,
	    		formatter: "DateTime",
	    	},
	    	{property: "jobEndTime",
	    		label: getText("rules.rulesPage.estimations.table.column.endTime.txt"),
	    		width: 160,
	    		formatter: "DateTime",
	    	},
	    	{property: "createUser",
	    		label: getText("rules.rulesPage.estimations.table.column.createdBy.txt"),
	    		width: 150,
	    		formatter: 0,
	    	}
    	]
    },

	getTableKey: function() {
   		return "Main";
	},

    _bShowPanels: false,

	updateToolbar: function() {
		const controller = this.getController();
        const toolbar = this.getToolbar();

		if (toolbar) {
			toolbar.destroyContent();

			const toolbarId = toolbar.getId();
			const allButton = new sas.hc.m.Button({
   				id: toolbarId + "UserFilterAllButton",
   				text: getText("rules.rulesPage.rules.actions.all.txt"),
                press: [controller.handleUserFilterAllPress, controller],
	        } );
			const mineButton = new sas.hc.m.Button({
   				id: toolbarId + "UserFilterMineButton",
	            text: getText("rules.rulesPage.rules.actions.mine.txt"),
                press: [controller.handleUserFilterMinePress, controller],
	        } );

			// Toggle All/User's estimations
			toolbar.addContent(new sas.hc.m.SegmentedButton({
                id: toolbarId + "UserFilterButton",
	            buttons: [allButton, mineButton],
	            selectedButton: controller.isShowAllEstimations() ? allButton : mineButton,
	        } ).addStyleClass("cfFirstToolbarItem") );

			// Create estimation
            toolbar.addContent(new sas.hc.m.Button({
                id: toolbarId + "NewButton",
                icon: sas.icons.HC.NEWSTARBURST,
                tooltip: getText("fcm.overview.new.txt"),
                press: [controller.handleNewPress, controller],
            } ) );
            // View result
            toolbar.addContent(new sas.hc.m.Button({
                id: toolbarId + "ResultsButton",
                icon: sas.icons.HC.FILE,
                tooltip: getText("rules.ruleEstimatesDetailMenu.viewResults.txt"),
                press: [this.handleViewResultsPress, this],
                enabled: false,
            } ) );
            // View log
            toolbar.addContent(new sas.hc.m.Button({
                id: toolbarId + "LogButton",
                icon: sas.icons.HC.LOG,
                tooltip: getText("rules.estimation.log.input.label"),
                press: [controller.showLog, controller],
                enabled: false,
            } ) );
            /* TBD in 6.2
            // Duplicate
            toolbar.addContent(new sas.hc.m.Button({
                id: toolbarId + "DuplicateButton",
                icon: sas.icons.HC.COPY,
                tooltip: getText("fcm.overview.duplicate.txt"),
                press: this.handleDuplicatePress,
                enabled: false,
            } ) );
            */
            // Cancel
            toolbar.addContent(new sas.hc.m.Button({
                id: toolbarId + "CancelButton",
                icon: sas.icons.HC.CANCEL,
                tooltip: getText("fcm.application.cancel.txt"),
                press: [this.handleCancelPress, this],
                enabled: false,
            } ) );
            // Delete
            toolbar.addContent(new sas.hc.m.Button({
                id: toolbarId + "DeleteButton",
                icon: sas.icons.HC.DELETE,
                tooltip: getText("fcm.overview.delete.txt"),
                press: [this.handleDeletePress, this],
                enabled: false,
            } ) );
            /* TBD in 6.2
            // Print
            toolbar.addContent(new sas.hc.m.Button({
                id: toolbarId + "PrintButton",
                icon: sas.icons.HC.PRINT,
                tooltip: getText("fcm.application.print.txt"),
                press: this.handlePrintPress,
                enabled: false,
            } ) );
            */
            // Properties
            toolbar.addContent(new sas.hc.m.Button({
                id: toolbarId + "PropertiesButton",
                icon: sas.icons.HC.PROPERTIES,
                tooltip: getText("fcm.application.properties.txt"),
                press: [controller.showProperties, controller],
                enabled: false,
            } ) );
            // Refresh
			toolbar.addContent(new sas.hc.m.Button({
			    id: toolbarId + "RefreshButton",
			    icon: sas.icons.HC.REFRESH,
			    tooltip: getText("fcm.application.refresh.txt"),
			    press: [controller.handleRefreshPress, controller],
			} ) );
		}
	},

	ButtonIdAndButtonFilterMapping: null,
	getButtonIdAndButtonFilterMapping: function () {
		if (this.ButtonIdAndButtonFilterMapping === null) { // lazy initialization because IE11 doesn't support creating a map with an iterable!
			const map = new Map();
			map.set("ResultsButton", new ButtonEnabler(SelectionMode.SINGLE, function (estimation) {
				return estimation.isResultAvaiable;
			} ) );
			map.set("LogButton",     new ButtonEnabler(SelectionMode.SINGLE, function (estimation) {
				return estimation.isLogAvaiable;
			} ) );
			map.set("DuplicateButton",   new ButtonEnabler(SelectionMode.SINGLE));
			map.set("CancelButton",      new ButtonEnabler(SelectionMode.SINGLE, function (estimation) {
				return estimation.isCanBeCancelled;
			} ) );
			map.set("DeleteButton",      new ButtonEnabler(SelectionMode.MULTIPLE, function (estimation) {
				return estimation.isCanBeDeleted;
			} ) );
			map.set("PrintButton",       new ButtonEnabler(SelectionMode.SINGLE));
			map.set("PropertiesButton",  new ButtonEnabler(SelectionMode.SINGLE));
			this.ButtonIdAndButtonFilterMapping = map;
		}
		return this.ButtonIdAndButtonFilterMapping;
	},

	enableToolbarButtons: function (toolbarId, selectedEstimations) {
		this.getButtonIdAndButtonFilterMapping().forEach(function (buttonFilter, buttonId) {
			const button = sap.ui.getCore().byId(toolbarId + buttonId);
			if (button) {
				const toEnable = buttonFilter.isToEnable(selectedEstimations);
				button.setEnabled(toEnable);
			}
		}, this );
	},

	updateToolbarEnablement: function (selectedEstimations) {
		const toolbarId = this.getToolbar().getId();
		this.enableToolbarButtons(toolbarId, selectedEstimations);
	},

	updatePanels: function() {
	},

	getTableUrl: function(searchTerm) {
		return this.getController().getTableUrl(searchTerm);
	},

	doDefaultAction: function() {
		this.getController().performDefaultAction();
	},

    getCountFormatter: function (val) {
		return (val) ? Math.max(0, val) : 0;
    },

	getDateTimeFormatter: function (val) {
		if (!val) {
			return val;
		}
		const date = new Date(val);
		if (isNaN(date.getTime())) {
			return val;
		}
		return sas.fscf.dateTimeFormatter.format(date, true);	// true means use UTC
	},

    getStatusFormatter: function () {
        return function (val) {
            if (val) {                                                      // Not sure why val is undefined sometimes.
                const status = val.toLowerCase().trim().replace(/ /g, "."); // Account for "start failed"
                const messageKey = "rules.ruleEstimate.status." + status + ".txt";
                return getText(messageKey);
            }
        };
    },

	getTextFormatter: function(type) {
		switch (type) {
			case "DateTime":
                return this.getDateTimeFormatter;
            case "Count":
                return this.getCountFormatter;
            case "Status":
                return this.getStatusFormatter();
			default:
				return function (val) {
					return val;
				};
		}
	},

    handleDuplicatePress: function () {
  		sas.fscf.removeAllMessageStrips();
    },

    getSelectedEstimationIndex: function () {
    	return this.getTable().getSelectedIndex();
    },

    getSelectedEstimationIndexes: function () {
    	return this.getTable().getSelectedIndices();
    },

    handlePrintPress: function () {
  		sas.fscf.removeAllMessageStrips();
    },

    getToolbar: function () {
    	return sap.ui.getCore().byId(this.getId() + "Toolbar");
    },

    handleCancelPress: function () {
  		sas.fscf.removeAllMessageStrips();
    	// "this" is bound to the view itself, not the target of the event.
    	const controller = this.getController();
    	controller.handleMenuItemPress(controller.getOpCode().CANCEL, this.getSelectedEstimationIndex());
    },

    confirmEstimationCancel: function (estimation, cancelEstimationCallback) {
    	const cancelEstimation = cancelEstimationCallback.bind(this.getController(), estimation);

        sas.hc.m.MessageDialog.warning({
            title: getText("rules.estimation.cancel.estimation.title"),
            text: getText("rules.estimation.cancel.prompt.msg"),
            buttonActions: [
                sas.hc.m.MessageDialogAction.Ok,
                {text: sas.hc.m.MessageDialogAction.Cancel,
                	defaultButton: true,
                	initialFocus: true,
                },
            ],
            buttonPress: function(event) {
            	this.getButtons().forEach(function (each) {
            		each.setEnabled(false);
            	} );
            	const button = event.getParameter("origin");
            	if (button.getMessageDialogAction() === sas.hc.m.MessageDialogAction.Ok) {
            		cancelEstimation();
            	}
            },
        } );
    },

    handleDeletePress: function () {
        sas.fscf.removeAllMessageStrips();
		// "this" is bound to the view itself, not the target of the event.
		const controller = this.getController();
		controller.handleMenuItemPress(controller.getOpCode().DELETE, this.getSelectedEstimationIndex());
  	},

    confirmEstimationsDelete: function (deleteEstimationsCallback) {
         sas.hc.m.MessageDialog.warning({
            text: getText("rules.estimation.delete.prompt.msg"),
            title: getText("rules.estimation.delete.dialog.title"),
            buttonActions: [
                sas.hc.m.MessageDialogAction.Delete,
                {text: sas.hc.m.MessageDialogAction.Cancel,
                	defaultButton: true,
                	initialFocus: true,
                },
            ],
            buttonPress: function(event) {
            	this.getButtons().forEach(function (each) {
            		each.setEnabled(false);
            	} );
            	const button = event.getParameter("origin");
            	if (button.getMessageDialogAction() === sas.hc.m.MessageDialogAction.Delete) {
            		deleteEstimationsCallback();
            	}
            },
        } );
    },

    enableContextMenu: false,   // Context menu is NOT allowed to be shown for prod.!!!
    setUpEstimationListContextMenu: function () {
        if (!this.enableContextMenu) {
            return;
        }

    	const NOT_SPECIFIED = -1;
		const controller = this.getController();
		let selectedEstimationIndex = NOT_SPECIFIED;

		const handleMenuItemPress = function (event) {
			if (selectedEstimationIndex !== NOT_SPECIFIED) {
				const menuItem = event.getParameter("item");
				if (menuItem !== null) {
					controller.handleMenuItemPress(menuItem.code, selectedEstimationIndex);
				}
			}
		};

        // Create the top-level menu that will be shown upon context menu activation
        const estimationList = this.getTable();

        estimationList.attachCellContextmenu(function (event) {
        	const selectedRowIndex = parseInt(event.getParameter("rowIndex"));
        	if (!isNaN(selectedRowIndex)) {
        		selectedEstimationIndex = selectedRowIndex;
        	}
        } );

        estimationList.attachBrowserEvent("contextmenu", function (event) {
        	selectedEstimationIndex = NOT_SPECIFIED;
        	// Don't show the context menu, if the right-click occurred in a cell with the selection checkbox.
        	// In this case, depending on where the click occurred within the cell (over a checkbox, or area
        	// surrounding the checkbox, className can be one of the following: "sapMCbBg sapMCbHoverable
        	// sapMCbMark" of "sapMCb".
        	if (event.target.className.indexOf("sapMCb") !== -1) {
				event.preventDefault();
				event.stopPropagation();
        	}
        } );

        // Menu items attributes.
        const OpCode = this.getController().getOpCode();
        const menuItemsAttributes = [
        	{label: getText("rules.ruleEstimatesDetailMenu.viewResults.txt"), code: OpCode.VIEW_RESULTS},
        	{label: getText("rules.estimation.log.input.label"), code: OpCode.VIEW_LOG},
        	{label: getText("fcm.application.cancel.txt"), code: OpCode.CANCEL},
        	{label: getText("fcm.overview.delete.txt"), code: OpCode.DELETE},
			// {label: getText("fcm.overview.copy.txt"), code: OpCode.COPY},  // TBD in 6.2
			{label: getText("fcm.application.properties.txt"), code: OpCode.VIEW_PROPERTIES},
        ];
        // Function to create and add menu items to menu.
        const addMenuItemsToMenu = function (menu, menuItemsAttrs) {
        	menuItemsAttrs.forEach(function (each) {
                const item = new sas.hc.ui.unified.MenuItem({
                    text: each.label,
                    select: handleMenuItemPress,
                } );
                item.code = each.code;
                menu.addItem(item);
        	} );
        };
        // Create menu.
        const menu = new sas.hc.ui.unified.Menu();
        // Create menu items and add them to the menu.
        addMenuItemsToMenu(menu, menuItemsAttributes);
        // Attach the menu to the estimation table.
        estimationList.setAggregation("contextMenu", menu);
    },

    handleViewResultsPress: function () {
  		sas.fscf.removeAllMessageStrips();
    	// "this" is bound to the view itself, not the target of the event.
    	const controller = this.getController();
    	controller.handleMenuItemPress(controller.getOpCode().VIEW_RESULTS, this.getSelectedEstimationIndex());
    },

    toast: function (toast) {
    	sas.hc.m.MessageToast.show(toast, {
    		animationDuration: 3000,
    		duration: 10000,
    		width: "40rem",
    	} );
    },

    error: function (title, error, errorDetail) {
    	sas.hc.m.MessageDialog.error({
            text: error,
            title: title,
            detailText: errorDetail,
        } );
    },

    info: function (title, info) {
        sas.hc.m.MessageDialog.information({
            text: info,
            title: title,
        } );
    },

    getRulePageContainerId: function () {
        return this.getParent().sId; // convoluted...but sas.fscf.showMessageStrip() requires it!
    },

    showErrorMessage: function (text) {
        sas.fscf.showErrorMessageStrip(text, this.getRulePageContainerId());
        return this;
    },

    showSuccessMessage: function (text) {
        sas.fscf.showSuccessMessageStrip(text, this.getRulePageContainerId());
        return this;
    },

    showWarningMessage: function (text) {
        sas.fscf.showWarningMessageStrip(text, this.getRulePageContainerId());
        return this;
    },
} );
