jQuery.sap.require("sas.fscf.views.estimation.HcHelper");
jQuery.sap.require("sas.fscf.views.estimation.EstimationHelper");
jQuery.sap.declare("sas.fscf.views.estimation.EstimationController");

const HcHelper = sas.fscf.views.estimation.HcHelper;
const TableAdapter = HcHelper.TableAdapter;
const EstimationHelper = sas.fscf.views.estimation.EstimationHelper;
const MessageType = EstimationHelper.MessageType;
const RequestType = EstimationHelper.RequestType;
const RuleHelper = EstimationHelper.RuleHelper;
const UrlBuilder = EstimationHelper.UrlBuilder;
const downloadFile = EstimationHelper.downloadFile;
const getLocalizedRuleState = EstimationHelper.getLocalizedRuleState;
const getText = EstimationHelper.getText;
const logError = EstimationHelper.logError;

sap.ui.core.mvc.Controller.extend("sas.fscf.views.estimation.EstimationController", {
	DialogId: {
		CODE: "EstimationCodeDialog",
        DEBUG_MESSAGES: "DebugMessagesDialog",
        LOG: "LogDialog",
        MATCHED_TRANSACTIONS: "MatchedTransactionsDialog",
        PROPERTIES: "EstimationPropertiesDialog",
        RULE: "EstimationRuleDialog",
		TIMING_METRICS: "EstimationTimingMetricsDialog",
    },

    estimationsController: null, // An instance of RulesPageEstimations.Controller

    deleteEstimation: function (callback) {
        return this.estimationsController.deleteEstimations(callback);
    },

    getAlertType: function (alertId) {
        return this.estimationsController.getAlertType(alertId);
    },

    getBizUnit: function () {
        return this.estimationsController.getBusinessUnitId();
    },

    getEstimation: function () {
        return this.estimationsController.getSelectedEstimation();
    },

	createDialog: function (viewName) {
        const id = this.estimationsController.getView().sId + viewName;
        const qualifiedViewName = "sas.fscf.views.estimation." + viewName;

		return sap.ui.jsfragment(id, qualifiedViewName, this); // 3 params: id, view name, controller
	},


	// BEGIN: Estimation Properties dialog
	propertiesDialog: null,
	propertiesModel: null,

	getPropertiesDialog: function () {
		if (this.propertiesDialog === null) {
			this.propertiesDialog = this.createPropertiesDialog();
		}
		return this.propertiesDialog;
	},

	createPropertiesDialog: function () {
		return this.createDialog(this.DialogId.PROPERTIES);
	},

	showProperties: function (aBizUnitId, anEstimation, refreshEstimationsCallback) {
        const estimation = (anEstimation) ? anEstimation : this.getEstimation();
        if (!estimation) {
            return null;
        }

        const bizUnitId = (aBizUnitId) ? aBizUnitId : this.getBizUnit();
		const url = UrlBuilder.getEstimationSummaryUrl(bizUnitId, estimation.id);
		const ajaxParams = {
			url: url,
			type: RequestType.GET,
			headers:sas.fscf.getAjaxHeaders()
		};
		const self = this;
		sas.ajax(ajaxParams)
			.done(function (estimationSummary) {
				self.propertiesModel = new sap.ui.model.json.JSONModel({estimation: estimationSummary});
                self.doShowProperties();
                // If the status was Finished, it might have changed to Completed.
				// Update the status and reflect the change in the Estimation List.
                estimation.status = estimationSummary.resultSummary.status;
                if (refreshEstimationsCallback) {
                    refreshEstimationsCallback();
                }
			} )
			.fail(function (response) {
				const message = getText("rules.estimation.retrieve.properties.error.msg");
                self.estimationsController.getView().showErrorMessage(message);
                const errorDetail = (response.responseJSON) ? response.responseJSON.message : null;
                logError(message, errorDetail, estimation);
			} );
	},

	doShowProperties: function () {
		this.getPropertiesDialog().open();
	},

	handlePropertiesDialogBeforeOpen: function (event) {
		event.getSource().setModel(this.propertiesModel);
	},

	handlePropertiesDialogAfterClose: function (event) {
		event.getSource().destroy();
		this.propertiesDialog = null;
	},
	// END: Estimation Properties dialog

	// BEGIN: Estimation Debug Messages dialog
    debugMessagesDialog: null,
    debugMessagesModel: null,

	createDebugMessagesDialog: function () {
		return this.createDialog(this.DialogId.DEBUG_MESSAGES);
	},

	getDebugMessagesDialog: function () {
		if (this.debugMessagesDialog === null) {
			this.debugMessagesDialog = this.createDebugMessagesDialog();
		}
		return this.debugMessagesDialog;
	},

	showDebugMessages: function (requestorView, logNumber) {
        sas.fscf.removeAllMessageStrips();
        requestorView.setBusy(true);

        const estimation = this.getEstimation();
        if (!estimation) {
            return null;
        }
		const url = UrlBuilder.getLogMessagesUrl(this.getBizUnit(), estimation.id, logNumber);
		const ajaxParams = {
			url: url,
			type: RequestType.GET,
			headers:sas.fscf.getAjaxHeaders()
        };

		const self = this;
		sas.ajax(ajaxParams)
			.done(function (logMessages) {
				self.debugMessagesModel = new sap.ui.model.json.JSONModel({log: logMessages});
                self.doShowDebugMessages(requestorView, logNumber);
			} )
			.fail(function (response) {
				const message = getText("rules.estimation.retrieve.log.error.fmt.msg", [estimation.name, estimation.id]);
				requestorView.showErrorMessage(message);
                const errorDetail = (response.responseJSON) ? response.responseJSON.message : null;
                logError(message, errorDetail, estimation);
            } )
            .always(function () {
                requestorView.setBusy(false);
            } );
	},

    filterMessages: function (log) {
        log.debugMessages = [];
        log.debugEstimationMessages = [];

        if (!log.logEvents)  {
        	return;
        }

        const debugMessages = log.logEvents.filter(function (each) {
            each.type = MessageType.valueOf(each.level);
            return each.type === MessageType.DEBUG;
        } );

        debugMessages.forEach(function (each) {
            if (each.message.includes(MessageType.DEBUG_ESTIMATION_INDICATOR)) {
                each.type = MessageType.DEBUG_ESTIMATION;
                log.debugEstimationMessages.push(each);
            } else {
                log.debugMessages.push(each);
            }
        } );
    },

    setUpDebugMessages: function (log, logNumber) {
        log.estimation = this.getEstimation();
        if (logNumber) {
            log.logNumber = logNumber;
        }
        this.filterMessages(log);
    },

	doShowDebugMessages: function (requestorView, logNumber) {
        const log = this.debugMessagesModel.getData().log;
        this.setUpDebugMessages(log, logNumber);

        if ((this.getEstimation().threadCount === 0)
                && (log.debugEstimationMessages.length === 0) && (log.debugMessages.length === 0) ) {
            sas.hc.m.MessageDialog.information({
                title: getText("rules.estimation.debug.messages.title"),
                text: getText("rules.esimtation.there.are.no.messages.msg"),
            } );
            return;
        } else {
            this.getDebugMessagesDialog()
                .setRequestorView(requestorView)
                .open();
        }
	},

	handleDebugMessagesDialogBeforeOpen: function (event) {
        event.getSource().setModel(this.debugMessagesModel);
    },

    handleDebugMessagesDialogAfterClose: function (event) {
		event.getSource().destroy();
		this.debugMessagesDialog = null;
	},
	// END: Estimation Debug Messages Dialog

    // BEGIN: Estimation Log dialog
    logDialog: null,
    logModel: null,

	createLogDialog: function () {
		return this.createDialog(this.DialogId.LOG);
	},

	getLogDialog: function () {
		if (this.logDialog === null) {
			this.logDialog = this.createLogDialog();
		}
		return this.logDialog;
	},

	showLog: function (requestorView, logNumber) {
        sas.fscf.removeAllMessageStrips();
        requestorView.setBusy(true);

        const estimation = this.getEstimation();
        if (!estimation) {
            return null;
        }
		const url = UrlBuilder.getLogUrl(this.getBizUnit(), estimation.id, logNumber);
		const ajaxParams = {
			url: url,
			type: RequestType.GET,
			headers:sas.fscf.getAjaxHeaders()
        };
		const self = this;
        const showLog = function (log) {
            self.logModel = new sap.ui.model.json.JSONModel({log: {logRendered: log}});
            self.doShowLog(requestorView, logNumber);
        };

		$.ajax(ajaxParams)
			.done(function (log) {
				showLog(log);
			} )
			.fail(function (response) {
                if (response.status === 200) { // false alarm
                    showLog(response.responseText);
                } else {
                    const message = getText("rules.estimation.retrieve.log.error.fmt.msg", [estimation.name, estimation.id]);
                    requestorView.showErrorMessage(message);
                    const errorDetail = (response.responseJSON) ? response.responseJSON.message : null;
                    logError(message, errorDetail, estimation);
                }
            } )
            .always(function () {
                requestorView.setBusy(false);
            } );
	},

    setUpLog: function (log, logNumber) {
        log.estimation = this.getEstimation();
        if (logNumber) {
            log.logNumber = logNumber;
        }
    },

	doShowLog: function (requestorView, logNumber) {
        this.setUpLog(this.logModel.getData().log, logNumber);
        this.getLogDialog()
                .setRequestorView(requestorView)
                .open();
	},

	handleLogDialogBeforeOpen: function (event) {
        event.getSource().setModel(this.logModel);
    },

    handleLogDialogAfterClose: function () {
		this.logDialog = null;
    },
	// END: Estimation Log Dialog

	// BEGIN: Timing Metrics dialog
	timingMetricsDialog: null,
	timingMetricsModel: null,

	getTimingMetricsDialog: function () {
		if (this.timingMetricsDialog === null) {
			this.timingMetricsDialog = this.createTimingMetricsDialog();
		}
		return this.timingMetricsDialog;
	},

	createTimingMetricsDialog: function () {
		return this.createDialog(this.DialogId.TIMING_METRICS);
	},

	showTimingMetrics: function (resultsView) {
        const estimation = this.getEstimation();
        if (!estimation) {
            return null;
        }
		const url = UrlBuilder.getTimingMetricsUrl(this.getBizUnit(), estimation.id);
		const ajaxParams = {
			url: url,
			type: RequestType.GET,
			headers:sas.fscf.getAjaxHeaders()
        };

		const self = this;
		sas.ajax(ajaxParams)
			.done(function (timingMetrics) {
				self.timingMetricsModel = new sap.ui.model.json.JSONModel({estimation: timingMetrics});
				self.doShowTimingMetrics();
			} )
			.fail(function (response) {
				const message = getText("rules.estimation.retrieve.timing.metrics.error.msg");
				resultsView.showErrorMessage(message);
                const errorDetail = (response.responseJSON) ? response.responseJSON.message : null;
                logError(message, errorDetail, estimation);
			} );
	},

    setUpTimingMetrics: function (timingMetrics) {
        timingMetrics.forEach(function (each) {
            RuleHelper.setDisplayId(each.rule);
         } );
    },

	doShowTimingMetrics: function () {
        const timingMetrics = this.getTimingMetricsModel().getData().estimation.timingMetrics;

        this.setUpTimingMetrics(timingMetrics);
		this.getTimingMetricsDialog().open();
	},

	handleTimingMetricsDialogBeforeOpen: function (event) {
        event.getSource().setModel(this.timingMetricsModel);
	},

	handleTimingMetricsDialogAfterClose: function (event) {
        event.getSource().destroy();
		this.timingMetricsDialog = null;
	},
	// END: Timing Metrics dialog

    // BEGIN: Rule dialog
	ruleDialog: null,
	ruleModel: null,

	getRuleDialog: function () {
		if (this.ruleDialog === null) {
			this.ruleDialog = this.createRuleDialog();
		}
		return this.ruleDialog;
	},

	createRuleDialog: function () {
		return this.createDialog(this.DialogId.RULE);
	},

    showRule: function (selectedRule, resultsView) {
        resultsView.setBusy(true);

        const estimation = this.getEstimation();
        if (!estimation) {
            return null;
        }

        const bizUnitId = this.getBizUnit();
		const url = UrlBuilder.getRuleUrl(bizUnitId, estimation.id, selectedRule.id);
		const ajaxParams = {
			url: url,
			type: RequestType.GET,
			headers:sas.fscf.getAjaxHeaders()
		};
		const self = this;
		sas.ajax(ajaxParams)
			.done(function (rule) {
				self.ruleModel = new sap.ui.model.json.JSONModel({rule: rule});
                self.doShowRule(rule);
			} )
			.fail(function (response) {
				const message = getText("rules.estimation.retrieve.rule.error.msg");
                resultsView.showErrorMessage(message);
                const errorDetail = (response.responseJSON) ? response.responseJSON.message : null;
                logError(message, errorDetail, estimation);
			} )
            .always(function () {
                resultsView.setBusy(false);
            } );
    },

    setUpRule: function (rule) {
        rule.alertType = this.estimationsController.getAlertType(rule.alertTypeId);
        RuleHelper.setDisplayId(rule);
        rule.isAlertRule = (rule.alertTypeId !== "");
        rule.stateLocalized = getLocalizedRuleState(rule.state);

        let transactionScopeString = (rule.multiOrgNodeDisplayName) ? rule.multiOrgNodeDisplayName : "";
        transactionScopeString += (rule.multiOrgNodeKey) ? " (" + rule.multiOrgNodeKey + ")" : "";
        rule.transactionScopeQualified = transactionScopeString;

        let transactionTypeString = (rule.transactionTypeDisplayName) ? rule.transactionTypeDisplayName : "";
        transactionTypeString += (rule.transactionTypeAcronym) ? " (" + rule.transactionTypeAcronym + ")" : "";
        rule.transactionTypeQualified = transactionTypeString;
    },

    doShowRule: function (rule) {
        this.setUpRule(rule);
		this.getRuleDialog().open();
	},

	handleRuleDialogBeforeOpen: function (event) {
		event.getSource().setModel(this.ruleModel);
	},

	handleRuleDialogAfterClose: function (event) {
		event.getSource().destroy();
		this.ruleDialog = null;
	},
    // END: Rule dialog

    // BEGIN: Matched transactions dialog
    matchedTransactionsDialog: null,
    matchedTransactionsModel: null,

	createMatchedTransactionsDialog: function () {
		return this.createDialog(this.DialogId.MATCHED_TRANSACTIONS);
	},

	getMatchedTransactionsDialog: function () {
		if (this.matchedTransactionsDialog === null) {
			this.matchedTransactionsDialog = this.createMatchedTransactionsDialog();
		}
		return this.matchedTransactionsDialog;
    },

	showMatchedTransactions: function (ruleTable, idx, showAlerts) {
        const selectedRuleMetrics = new TableAdapter(ruleTable).getObject(idx);
        const data = {
            estimation: this.getEstimation(),
            ruleDecisionMetrics: selectedRuleMetrics,
            forAlerts: showAlerts,
        };

        this.matchedTransactionsModel = new sap.ui.model.json.JSONModel({"data": data});
		this.doShowMatchedTransactions();
	},

	doShowMatchedTransactions: function () {
        this.getMatchedTransactionsDialog()
            .setModel(this.matchedTransactionsModel)
            .open();
    },

    exportTransactionsToExcel: function () {
        const myData = this.matchedTransactionsModel.getData().data;
        const estimation = myData.estimation;
        const rule = myData.ruleDecisionMetrics.rule;
        const fireType = myData.forAlerts ? "alerted" : "decisions";
        const uri = UrlBuilder.getExportToExcelUrl(estimation.businessUnitId, estimation.id, rule.id, fireType);

        downloadFile(uri);
    },

	handleMatchedTransactionsDialogBeforeOpen: function () {
    },

    handleMatchedTransactionsDialogAfterClose: function () {
		this.matchedTransactionsDialog.destroy();
		this.matchedTransactionsDialog = null;
    },
    // END: Matched transactions dialog

	// Public
	getPropertiesModel: function () {
		return this.propertiesModel;
    },

	getRuleModel: function () {
		return this.ruleModel;
	},

	getTimingMetricsModel: function () {
		return this.timingMetricsModel;
	},

    setEstimationsController: function (aController) {
        this.estimationsController = aController;
    },
} );
