jQuery.sap.require("sas.fscf.views.estimation.EstimationController");
jQuery.sap.require("sas.fscf.views.estimation.EstimationHelper");
jQuery.sap.require("sas.fscf.views.estimation.EstimationResultsHelper");

const EstimationHelper = sas.fscf.views.estimation.EstimationHelper;
const NumberAbbreviator = EstimationHelper.NumberAbbreviator;
const RuleHelper = EstimationHelper.RuleHelper;
const getText = EstimationHelper.getText;
const EstimationResultsHelper = sas.fscf.views.estimation.EstimationResultsHelper;
const AllAlertsBuilder = EstimationResultsHelper.AllAlertsBuilder;
const AllRulesBuilder = EstimationResultsHelper.AllRulesBuilder;

const formatter = EstimationHelper.getFormatter();

sap.ui.controller("sas.fscf.views.estimation.EstimationResults", {
	estimationController: null,  // An instance of EstimationController
	estimationsController: null, // An instance of RulesPageEstimations.controller
	estimationsViewId: null,
	alertsByQueueModel: null,
    ruleMetricsModel: null,

    decisionMetrics: null,
    getDecisionMetrics: function () {
        return this.decisionMetrics;
    },

    setDecisionMetrics: function (metrics) {
        this.decisionMetrics = metrics;
    },

    queueMetrics: null,
    getQueueMetrics: function () {
        return this.queueMetrics;
    },
    isQueueMetricsAvailable: function () {
        return (this.queueMetrics !== null) && (this.queueMetrics.length !== 0);
    },

    setQueueMetrics: function (metrics) {
        this.queueMetrics = metrics;
    },

    ruleMetrics: null,
    getRuleMetrics: function () {
        return this.ruleMetrics;
    },

    setRuleMetrics: function (metrics) {
        this.ruleMetrics = metrics;
    },

	getRules: function () {
		return this.ruleMetricsModel.getProperty("/ruleMetrics");
	},

	getRuleMetricsModel: function () {
		return this.ruleMetricsModel;
	},

	getSelectedRule: function () {
		const ruleTable = this.getView().getRuleTable();
		const selectedRuleMetrics = ruleTable.getSelectedIndices().map(function (each) {
			const context = ruleTable.getContextByIndex(each);
			const path = context.sPath;

			return this.ruleMetricsModel.getProperty(path);
		}, this );
		return (selectedRuleMetrics.length > 0) ? selectedRuleMetrics[0].rule : null;
	},

	doHandleRuleSelectionChange: function () {
        // no-op
        // Rule Evaluation is not supported in 6.1, maybe 6.2.
	},

    setEstimationsViewId: function (id) {
    	this.estimationsViewId = id;
	},

	setUpAlertsByQueueModel: function (alerts) {
		this.alertsByQueueModel = new sap.ui.model.json.JSONModel({alerts: alerts});
	},

	setUpRuleMetricsModel: function (ruleMetrics) {
		this.ruleMetricsModel = new sap.ui.model.json.JSONModel({ruleMetrics: ruleMetrics});
	},

	onInit: function () {
		const view = this.getView();

        view.setAlertsByQueueModel(this.alertsByQueueModel);
		view.setRuleTableModel(this.ruleMetricsModel);
	},

	enableRuleButton: function (rule) {
		const toEnable = (rule === null) ? false : !rule.isAllRules;
		this.getView().enableRuleButton(toEnable);
	},

	handleRuleSelectionChange: function () {
        const selectedRule = this.getSelectedRule();

        this.enableRuleButton(selectedRule); // S1506132
		if (selectedRule !== null) {
			this.doHandleRuleSelectionChange(selectedRule);
		}
	},

	getEstimationController: function () {
		if (!this.estimationController) {
            this.estimationController = new sas.fscf.views.estimation.EstimationController();
            this.estimationController.setEstimationsController(this.estimationsController);
        }
		return this.estimationController;
	},

	formatRate: function (rate) {
        const answer = (rate === 0)
	    	? 0
	    	: (rate < 1)
	    		? +rate.toFixed(3) // trim trailing zeros
	    		: (rate < 10)
	    			? +rate.toFixed(2)
                    : Math.round(rate);
        return formatter.formatNumber(answer);
	},

    setUpDecisionMetrics: function () {
        const decisionMetrics = this.getDecisionMetrics();

        // ADR
        const entityDecisionCounts = decisionMetrics.entityDecisionCounts;
        if (entityDecisionCounts !== undefined) {
            entityDecisionCounts.alerts = entityDecisionCounts.predictedConditionPositive;
            entityDecisionCounts.totalEntitiesWithFraud = (entityDecisionCounts.conditionPositive) // Not unavailable with pre-v6 estimations
                    ? entityDecisionCounts.conditionPositive
                    : 0;
            entityDecisionCounts.adr = this.formatRate(entityDecisionCounts.truePositiveRate * 100);
            entityDecisionCounts.adrString = entityDecisionCounts.adr + "%";
            entityDecisionCounts.adrCalculationString =
                    formatter.formatNumber(entityDecisionCounts.truePositive) + " / " + formatter.formatNumber(entityDecisionCounts.totalEntitiesWithFraud);
        } else {
        	decisionMetrics.entityDecisionCounts = {alerts: 0};
        }

        const transactionDecisionCounts = decisionMetrics.transactionDecisionCounts;
        transactionDecisionCounts.approveDecisions = transactionDecisionCounts.predictedConditionNegative;
        transactionDecisionCounts.nonApproveDecisions = transactionDecisionCounts.predictedConditionPositive;

        // TDR
        transactionDecisionCounts.totalFraudTransactions = (transactionDecisionCounts.conditionPositive)
        		? transactionDecisionCounts.conditionPositive
        		: 0;
        transactionDecisionCounts.tdr = this.formatRate(transactionDecisionCounts.truePositiveRate * 100);
        transactionDecisionCounts.tdrString = transactionDecisionCounts.tdr + "%";
        transactionDecisionCounts.tdrCalculationString =
                formatter.formatNumber(transactionDecisionCounts.truePositive) + " / " + formatter.formatNumber(transactionDecisionCounts.totalFraudTransactions);

        // VDR
        const transactionDecisionValues = decisionMetrics.transactionDecisionValues;
        if (transactionDecisionValues) { // transactionDecisionValues is not available for pre-v6 estimations
            transactionDecisionValues.vdr = (transactionDecisionValues.conditionPositive === 0)
                ? 0
                : this.formatRate(transactionDecisionValues.truePositive / transactionDecisionValues.conditionPositive * 100);
            transactionDecisionValues.vdrString = transactionDecisionValues.vdr + "%";
            transactionDecisionValues.vdrCalculationString =
                    formatter.formatNumber(transactionDecisionValues.truePositive) + " / " + formatter.formatNumber(transactionDecisionValues.conditionPositive);
        }

        // FPR - false positive ratio
        transactionDecisionCounts.falsePositiveRatioString = transactionDecisionCounts.falsePositive + ":" + transactionDecisionCounts.truePositive;
    },

    setUpQueueMetrics: function (queueMetrics) {
        if (this.isQueueMetricsAvailable()) {
            const allAlerts = new AllAlertsBuilder(queueMetrics).build();
            queueMetrics.splice(0, 0, allAlerts);

            queueMetrics.forEach(function (each) {
                each.queueId = (each.queueId).toString();
            } );
        }
    },

    setUpRuleMetricsRules: function (ruleMetrics) {
        ruleMetrics.forEach(function (each) {
            const rule = each.rule;
            if (!rule.isAllRules) {
                RuleHelper.setDisplayId(rule);
                rule.id = String(rule.id);
                rule.isAlertRule = (rule.alertTypeId !== undefined);
                if (rule.isAlertRule) {
                    rule.alertType = this.estimationsController.getAlertType(rule.alertTypeId);
                }
            }
        }, this );
    },

    // Must be called after setUpRuleMetricsRules() because this depends on the rule.isAlertRule property to be defined.
    setUpRuleMetricsEntityDecisions: function (ruleMetrics) {
        ruleMetrics.forEach(function (each) {
            const rule = each.rule;

            if (rule.isAlertRule || rule.isAllRules) {
                const entityDecisionCounts = each.entityDecisionCounts;
                // Alert Entities count.
                entityDecisionCounts.alertEntitiesCount = entityDecisionCounts.predictedConditionPositive;
                entityDecisionCounts.alertEntitiesCountAbbreviated = NumberAbbreviator.format(entityDecisionCounts.alertEntitiesCount);
                entityDecisionCounts.alertEntitiesCountLocalized = formatter.formatNumber(entityDecisionCounts.alertEntitiesCount);
                // FPR - false positive ratio
                entityDecisionCounts.falsePositiveRatioString = entityDecisionCounts.falsePositiveRatio[0] + ":" + entityDecisionCounts.falsePositiveRatio[1];
                // ADR - alert detection rate
                entityDecisionCounts.adr = this.formatRate(entityDecisionCounts.truePositiveRate * 100);
                entityDecisionCounts.adrString = entityDecisionCounts.adr + "%";
                entityDecisionCounts.adrCalculationString =
                        formatter.formatNumber(entityDecisionCounts.truePositive) + " / " + formatter.formatNumber(entityDecisionCounts.conditionPositive);
            } else {
                each.entityDecisionCounts = {
                    truePositive: 0,
                    falseNegative: 0,
                    falsePositive: 0,
                    conditionPositive: 0,
                    predictedConditionPositive: 0,
                    alertEntitiesCount: 0,
                    falseNegativeRate: 0.0,
                    truePositiveRate: 0.0,
                    totalString: "",
                };
            }
        }, this );
    },

    setUpRuleMetricsTransactionDecisionValues: function (ruleMetrics) {
        ruleMetrics.forEach(function (each) {
            const trxDecisionValues = each.transactionDecisionValues;
            // The amount of Transaction Decisions.
            trxDecisionValues.transactionDecisionsAmountAbbreviated = NumberAbbreviator.format(trxDecisionValues.predictedConditionPositive);
            trxDecisionValues.transactionDecisionsAmountLocalized = formatter.formatNumber(trxDecisionValues.predictedConditionPositive);

            // The amount of Total Processed Transactions
            trxDecisionValues.transactionsProcessedAmountAbbreviated = NumberAbbreviator.format(trxDecisionValues.totalPopulation);
            trxDecisionValues.transactionsProcessedAmountLocalized = formatter.formatNumber(trxDecisionValues.totalPopulation);

            // The amount of Transcations Fraud
            trxDecisionValues.transactionsFraudAmountAbbreviated = NumberAbbreviator.format(trxDecisionValues.conditionPositive);
            trxDecisionValues.transactionsFraudAmountLocalized = formatter.formatNumber(trxDecisionValues.conditionPositive);
        } );
    },

    setUpRuleMetricsTransactionDecisionCounts: function (ruleMetrics) {
        ruleMetrics.forEach(function (each) {
            const trxDecisionCounts = each.transactionDecisionCounts;
            // Transaction Decisions count.
            trxDecisionCounts.transactionDecisionsCountAbbreviated = NumberAbbreviator.format(trxDecisionCounts.predictedConditionPositive);
            trxDecisionCounts.transactionDecisionsCountLocalized = formatter.formatNumber(trxDecisionCounts.predictedConditionPositive);

            // Total Processed Transactions count
            trxDecisionCounts.transactionsProcessedCountAbbreviated = NumberAbbreviator.format(trxDecisionCounts.totalPopulation);
            trxDecisionCounts.transactionsProcessedCountLocalized = formatter.formatNumber(trxDecisionCounts.totalPopulation);

            // Transcations Fraud count
            trxDecisionCounts.transactionsFraudCountAbbreviated = NumberAbbreviator.format(trxDecisionCounts.conditionPositive);
            trxDecisionCounts.transactionsFraudCountLocalized = formatter.formatNumber(trxDecisionCounts.conditionPositive);

            // VDR - value detection rate
            trxDecisionCounts.vdr = this.formatRate(trxDecisionCounts.truePositiveRate * 100);
            trxDecisionCounts.vdrString = trxDecisionCounts.vdr + "%";
            trxDecisionCounts.vdrCalculationString =
                    formatter.formatNumber(trxDecisionCounts.truePositive) + " / " + formatter.formatNumber(trxDecisionCounts.conditionPositive);
            // FPR - false positive ratio
            trxDecisionCounts.falsePositiveRatioString = trxDecisionCounts.falsePositiveRatio[0] + ":" + trxDecisionCounts.falsePositiveRatio[1];
        }, this );
    },

    setUpRuleMetrics: function (ruleMetrics) {
        this.setUpRuleMetricsRules(ruleMetrics);
        this.setUpRuleMetricsEntityDecisions(ruleMetrics);
        this.setUpRuleMetricsTransactionDecisionValues(ruleMetrics);
        this.setUpRuleMetricsTransactionDecisionCounts(ruleMetrics);

        return ruleMetrics;
    },

    setUp: function (decisionMetrics, queueMetrics, ruleMetrics) {
        this.setDecisionMetrics(decisionMetrics);
        this.setUpDecisionMetrics();

        this.setQueueMetrics(queueMetrics);
        this.setUpQueueMetrics(queueMetrics);
        this.setUpAlertsByQueueModel(queueMetrics);

        const allRules = new AllRulesBuilder(Object.create({"decisionMetrics": decisionMetrics, "ruleMetrics": ruleMetrics})).build();
        ruleMetrics.splice(0, 0, allRules);
        this.setRuleMetrics(ruleMetrics);
        this.setUpRuleMetrics(ruleMetrics);
        this.setUpRuleMetricsModel(ruleMetrics);
    },

	showResults: function (decisionMetrics, queueMetrics, ruleMetrics, parentView) {
        this.setUp(decisionMetrics, queueMetrics, ruleMetrics);
        this.doShowResults(parentView);
        if (decisionMetrics.legacyResults) {
            this.getView().showWarningMessage(getText("rules.estimation.prior.release.missing.data.warning.msg"));
        }
	},

	doShowResults: function (parentView) {
		const view = sap.ui.view({
            id: parentView.sId + "EstimationResultsView",
			viewName: "sas.fscf.views.estimation.EstimationResults",
			controller: this,
            type: sap.ui.core.mvc.ViewType.JS,
		} );
		parentView.switchToEstimationResultsContent(view);
    },

    setEstimationsController: function (aController) {
    	this.estimationsController = aController;
    },
} );
