jQuery.sap.require("sas.fscf.views.estimation.EstimationHelper");

sap.ui.define("sas.fscf.views.estimation.EstimationResultsHelper",
	[],
	(function () {    /* jshint -W034 */
		"use strict"; /* jshint +W034 */

        const EstimationHelper = sas.fscf.views.estimation.EstimationHelper;
		const IdFactory = EstimationHelper.IdFactory;
        const NullCheck = EstimationHelper.NullCheck;
        const NumberAbbreviator = EstimationHelper.NumberAbbreviator;
        const SpacerFactory = EstimationHelper.SpacerFactory;
        const getText = EstimationHelper.getText;
        const parseInteger = EstimationHelper.parseInteger;

        const formatter = EstimationHelper.getFormatter();

 		const Icon = {
			ALERT: sas.icons.HC.ALERT,
            BLANK: sas.icons.HC.BLANK,
            BUG: sas.icons.HC.DEBUGGER,
			CHECKMARK_GREEN: sas.icons.HC.COMPLETESTATUS,
			CODE: sas.icons.HC.CODE,
			COPY: sas.icons.HC.COPY,
			CONTROL_EVIDENCE: sas.icons.HC.CONTROLEVIDENCE,
            CROSS_RED: sas.icons.HC.ERRORSTATUS,
            EXCEL: sas.icons.HC.EXCELFILE,
			DECISION: sas.icons.HC.DECISIONDIAGRAM,
			DELETE: sas.icons.HC.DELETE,
			DEPLOY: sas.icons.HC.DEPLOY,
			FLAG: sas.icons.HC.THRESHOLDGLOBAL,
			INFO: sas.icons.HC.INFORMATION,
			LOG: sas.icons.HC.LOG,
			METRICS: sas.icons.HC.METRICSTABLE,
			PERCENT: sas.icons.HC.PERCENTOFTOTALS,
			PRINT: sas.icons.HC.PRINT,
			PROPERTIES: sas.icons.HC.PROPERTIES,
			QUESTION_MARK: sas.icons.HC.HELPBUTTONTOOLBAR,
			SORT: sas.icons.HC.SORTGENERIC,
			SUM: sas.icons.HC.SUMSTATISTIC,
			TEST: sas.icons.HC.TEST,
			TIMING_METRICS: sas.icons.HC.PERFORMANCEINSIGHT,
			TRIANGLE: sas.icons.HC.CAUSALMODELS,
			WARNINGS: sas.icons.HC.WARNINGSTATUS,
			WARNINGS_AND_ERRORS: sas.icons.HC.VIEWWARNINGSANDERRORS,
			WINNER_RIBBON: sas.icons.HC.WINNER,
		};
		Object.freeze && Object.freeze(Icon);

		const IgnoreEvent = function (event) { // jQuery event, not sap.ui.base.Event
			event.stopPropagation();
			event.preventDefault();
		};

		const Popover = {
			CONTENT_WIDTH: "13rem",
			CONTENT_WIDTH_PLUS: "15rem",
			WIDTH: "14rem",
			WIDTH_PLUS: "16rem",
		};
		Object.freeze && Object.freeze(Popover);

		const newIconButton = function (properties) {
			const tooltip = (properties.tooltip !== undefined) ? properties.tooltip : " "; // Overwrite the default text from the icon.  An empty string won't do.

			return new sas.hc.m.Button({
				id: (properties.sId ? properties.sId : ""),
				icon: properties.icon,
				menuItemText: tooltip,
				tooltip: tooltip,
			} );
		};

		// @param headerInfo - Object with 2 properties: leftContent, rightContent (optional).  Both are Strings.
		const newPopoverHeader = function (headerInfo) {
			const answer = new sas.hc.m.Bar({
				design: sap.m.BarDesign.Header,
				contentLeft: [
					new sas.hc.m.Label({
						text: headerInfo.leftContent,
                        design: sap.m.LabelDesign.Bold,
                        wrapping: false,
					} ),
				],
			} ).addStyleClass("sapUiSmallMarginBeginEnd");
			if (headerInfo.rightContent !== undefined) {
				answer.addContentRight(new sas.hc.m.Label({
					text: headerInfo.rightContent,
                    design: sap.m.LabelDesign.Bold,
                    wrapping: false,
				} ) );
			}
			return answer;
		};

		const newPopoverSubheader = function (text) {
			return new sas.hc.m.HBox({
				alignItems: sap.m.FlexAlignItems.Center,
				justifyContent: sap.m.FlexJustifyContent.Start,
				items: [new sas.hc.m.FormattedText({
					htmlText: "<span style='font-size:smaller; color:gray'>" + text + "</span>"
				} ) ],
			} );
		};

		const newLabelTextSpacer = function () {
			return SpacerFactory.horizontalSpacer("0.25rem");
		};

		const LineRenderer = (function () {
			const doRender = function (properties) {
				return new sas.hc.m.HBox({
					id: IdFactory.create(properties.id),
					alignItems: sap.m.FlexAlignItems.Center,
					justifyContent: sap.m.FlexJustifyContent.Start,
					items: [
						SpacerFactory.horizontalSpacer((properties.topLevel ? "1.4rem" : "2.5rem")), // "1.6rem" : "2.8rem"
						newIconButton({icon: properties.icon}),
						new sas.hc.m.Label({
                            text: properties.label+":",
                            wrapping: false,
						} ),
						newLabelTextSpacer(),
						new sas.hc.m.Text({
                            text: properties.value,
                            tooltip: (properties.valueTooltip) ? properties.valueTooltip : "",
						} ),
					],
				} );
			};

			const render = function (param) {
				return Array.isArray(param)
					? param.map(function (each) {return doRender(each);})
					: doRender(param);
			};

			return {render: render};
		} )();

		const PopoverLineRenderer = (function () {
			const getLeftContent = function (properties) {
				const answer = [];
				if (properties.icon !== undefined) {
					answer.push(newIconButton({icon: properties.icon}));
                }
                const label = new sas.hc.m.Label({
                    text: properties.label,
                    design: ((properties.bold) ? sap.m.LabelDesign.Bold : sap.m.LabelDesign.Standard),
                    wrapping: false,
                } );
                if (properties.tooltip) {
                    label.setTooltip(properties.tooltip);
                }
				answer.push(label);
				return answer;
			};

            const newInfoButton = function () {
                const answer = (newIconButton({
                    icon: Icon.INFO,
                    sId: IdFactory.create("InfoButton"),
                } ) );
                answer.addEventDelegate({
                    onAfterRendering: function (event) {
                        event.srcControl.$().find("[id$='img']").addClass("infoButton");
                    }
                } );
                return answer;
            };

			const getRightContent = function (properties) {
				const items = [];
				if (properties.value !== undefined) {
					const valueText = new sas.hc.m.Text({
						id: IdFactory.create("PopoverValue"),
						text: properties.value,
					} );
					items.push(valueText);
				}
				if (properties.showInfoIcon) {
					items.push(newInfoButton());
				}
				if (properties.rightMargin === undefined) {
					items.push(SpacerFactory.horizontalSpacer(properties.rightMargin));
				}
				return new sas.hc.m.HBox({
					justifyContent: sap.m.FlexJustifyContent.END,
					alignItems: sap.m.FlexAlignItems.Center,
					items: [items],
				} );
			};

			const doRender = function (properties) {
				const items = [];
				items.push(
					new sas.hc.m.HBox({
						alignItems: sap.m.FlexAlignItems.Center,
						justifyContent: sap.m.FlexJustifyContent.Start,
						items: [getLeftContent(properties)],
					} )
				);
				const rightContent = getRightContent(properties);
				if (rightContent !== null) {
					items.push(rightContent);
				}
				return new sas.hc.m.HBox({
					id: IdFactory.create("DetailHBox"),
					alignItems: sap.m.FlexAlignItems.Center,
					justifyContent: sap.m.FlexJustifyContent.SpaceBetween,
					items: [items],
				} );
			};

			const render = function (param) {
				return Array.isArray(param)
					? param.map(function (each) {return doRender(each);})
					: doRender(param);
			};

			return{
				render: render,
			};
		} )();

		const newSectionTitleHBox = function (section) {
			return PopoverLineRenderer.render({
				label: section.title,
				value: section.titleRightContent,
			});
		};

		const newTileSectionsItems = function (tilePropertiesList) {
			return [SpacerFactory.verticalSpacer("0.3rem")] // "0.4rem"
				.concat(LineRenderer.render(tilePropertiesList));
		};

		// @param sectionInfo - Object with the following properties:
		//    title(String),
		//    total(String), positive({"<label>", "<amount>"}), negative({"<label>", "<amount>"}), or
		//    rate({calculation: String, result: String})
		const getSectionItems = function (sectionInfo) {
			const answer = [];
			const sectionContent = sectionInfo.content;

			if (sectionContent !== undefined) {
				if (sectionContent.rate !== undefined) {
					answer.push({label: sectionContent.rate.calculation,
                        value: sectionContent.rate.result,
                        icon: Icon.PERCENT,
					} );
				} else {
                    const positive = {label: sectionContent.positive.label,
                        value: sectionContent.positive.amount,
                        icon: Icon.CHECKMARK_GREEN,
                    };
                    const negative = {label: sectionContent.negative.label,
                        value: sectionContent.negative.amount,
                        icon: Icon.WARNINGS,
                    };
                    const isPositiveFirst = (sectionContent.positiveFirst !== undefined) ? sectionContent.positiveFirst : true;
                    answer.push(isPositiveFirst ? positive : negative);
                    answer.push(isPositiveFirst ? negative : positive);
                    answer.push({label: getText("rules.estimation.total.label"),
                    value: sectionContent.total,
                    icon: Icon.SUM,
                    } );
				}
			}
			return answer;
		};

		const newPopoverSectionItems = function (sectionInfo) {
			const isFirstSection = (sectionInfo.isFirst !== undefined) && sectionInfo.isFirst;
			const answer = [
				SpacerFactory.verticalSpacer(isFirstSection ? "0.3rem" : "0.8rem"),
				newSectionTitleHBox(sectionInfo),
				SpacerFactory.verticalSpacer("0.1rem"),
			];
			const sectionItemsRendition = PopoverLineRenderer.render(getSectionItems(sectionInfo));
			return answer.concat(sectionItemsRendition);
		};

		// @param properties -
		const newPopover = function (properties) {
			const content = new sas.hc.m.VBox({
				alignItems: sap.m.FlexAlignItems.Center,
				justifyContent: sap.m.FlexJustifyContent.Center,
			} );
			content.addEventDelegate({
				onAfterRendering: function (event) {
					const $popoverContent = event.srcControl.$();
					// Reduce the height of the HBoxes whose id has the suffix DetailHBox to pack them "tighter" to improve the visual hierarchy.
					$popoverContent.find("[id*='DetailHBox']").css("height", "22px");

					// Push the value in the popover up by 1 pixel to vertically-aligned with its corresponding label.
					$popoverContent.find("[id*='PopoverValue']").each(function () {
						const style = this.style;
						style.setProperty("line-height", "21px", "important");
						style.setProperty("vertical-align", "top", "important");
					} );
				},
			} );

			const popoverContent = [];
			if (properties.subheader !== undefined) {
                popoverContent.push(
                    newPopoverSubheader(properties.subheader),
                    SpacerFactory.verticalSpacer("0.5rem")
                );
			}
			Array.prototype.push.apply(popoverContent, properties.content);
			popoverContent.push(SpacerFactory.verticalSpacer("1.2rem"));
			const contentWidth = (properties.contentWidth !== undefined) ? properties.contentWidth : Popover.CONTENT_WIDTH;
			popoverContent.forEach(function (each) {
				each.setWidth(contentWidth);
				content.addItem(each);
			} );
			const answer = new sas.hc.m.ResponsivePopover({
				customHeader: properties.header,
				content: [content],
				modal: false,
                contentWidth: ((properties.width !== undefined) ? properties.width : Popover.WIDTH),
                placement: sap.m.PlacementType.HorizontalPreferredRight,
			} );
			answer.addStyleClass("cfEstRsltSummaryPopover");
			return answer;
		};

		const PopoverFactory = (function () {
			// Cache the popovers so that they can be referenced and destroyed when the dialog is destroyed.
			const popovers = [];

			const createPopover = function (properties) {
				return newPopover(properties);
			};

			const destroy = function () {
				popovers.forEach(function (each) {
					each.destroy();
				} );
			};

			return {
				destroy: destroy,
				new: createPopover,
			};
		} )();

		const TileToolbarBuilder = function () {
			let additionalInfo = null;
			let icon = null;
            let title = "";
            let tooltip = null;

			const setAdditionalInfo = function (info) {
				if ((info !== null) && (info !== undefined)) {
					additionalInfo = info;
				}
				return this;
			};

			const setIcon = function (anIcon) {
				if ((anIcon !== null) && (anIcon !== undefined)) {
					icon = anIcon;
				}
				return this;
			};

			const setTitle = function (aTitle) {
				if ((aTitle !== null) && (aTitle !== undefined)) {
					title = aTitle;
				}
				return this;
			};

			const setTooltip = function (text) {
				if ((text !== null) && (text !== undefined)) {
					tooltip = text;
				}
				return this;
			};

			const getProperties = function () {
				const answer = {
                    label: title,
                    tooltip: tooltip,
					bold: true,
				};
				if (icon !== null) {
					answer.icon = icon;
				}
				if (additionalInfo !== undefined) {
					answer.value = additionalInfo;
				}
				answer.showInfoIcon = true;
				answer.rightMargin = "0.75rem";
				return answer;
			};

			const build = function () {
				return PopoverLineRenderer.render(getProperties())
					.setWidth("100%")
					.addStyleClass("cfEstRsltTileToolbar");
			};

			return {
				build: build,
				setAdditionalInfo: setAdditionalInfo,
				setIcon: setIcon,
                setTitle: setTitle,
                setTooltip: setTooltip,
			};
		};

		/*
			@param properties - Json with 3 required properties
				title String
				icon
				content HBox or VBox
		*/
		const TileBuilder = function () {
			let content = null;
			let icon = null;
            let title = null;
            let tooltip = "";
			let additionalTitleInfo = null;
			let isGraphView = false;
			let graphViewSetUp = null;
			let popover = null;

			const setAdditionalInfo = function (aString) {
				additionalTitleInfo = aString;
				return this;
			};

			const setContent = function (aString) {
				content = aString;
				return this;
			};

			const setGraphViewSetUp = function (aFunction) {
				graphViewSetUp = aFunction;
				return this;
			};

			const setIcon = function (anIcon) {
				icon = anIcon;
				return this;
			};

			const setIsGraphView = function (aBoolean) {
				isGraphView = aBoolean;
				return this;
			};

			const setPopover = function (aPopover) {
				popover = aPopover;
				return this;
			};

			const setTitle = function (aString) {
				title = aString;
				return this;
            };

			const setTooltip = function (aString) {
				tooltip = aString;
				return this;
            };

			const adjustTile = function (tile) {
				const $tile = tile.$();
				// HC-override: reduce padding in header section.
				// Original: .sasMTileBaseContentHeaderSection {padding: .75rem;}
				$tile.find(".sasMTileBaseContentHeaderSection").css("padding", "0.3rem 0.75rem 0");

				// HC-override: remove all padding from the tile content section.
				// Original: .sasMTileBaseContentBodySection {padding: 0 .75rem .75rem .75rem;}
				$tile.find(".sasMTileBaseContentBodySection").css("padding", "0");

				const contentContainerId = tile.getAssociation("contentBodyCustomContentContainer");
				const $contentContainer = $tile.find("#"+contentContainerId);
				// 7.5 is the same as the graph view height and fit the donut well.  But we need 9.5rem in order to accommodate the
				// the Biz Metrics tile, otherwise, a vertical scrollbar appears.  Or we can set overflow to hidden.
				$contentContainer.css("height", "9.5rem");
			};

			const setUp = function (aTile) {
				aTile.addEventDelegate({
					onAfterRendering: function (event) {
						const tile = event.srcControl;
						adjustTile(tile);

						// Set up the Info button to hide/show the popover.
						const $tile = event.srcControl.$();
						const $infoButton = $tile.find("[id*='InfoButton']");
						const infoButton = $infoButton.control()[0];
						infoButton.attachPress(function () {
							popover.isOpen() ? popover.close() : popover.openBy(event.srcControl);
						} );

						// Ignore the mousedown event so that the user can't do marquee selection, which serves no purpose!
						const $canvas = $(event.srcControl.getDomRef()).find("canvas").first();
						$canvas.on("mousedown", IgnoreEvent);
					},
				} );
			};

			const getTileHeader = function () {
				return new TileToolbarBuilder()
                    .setTitle(title)
                    .setTooltip(tooltip)
					.setIcon(icon)
					.setAdditionalInfo(additionalTitleInfo)
					.build();
			};

			// Need to put the graph view in an HBox so that it can be centered horizontally within the content body of an OjbectTile.
			const getGraphViewContent = function () {
				return new sas.hc.m.HBox({
					width: "100%",
					height: "100%",
					justifyContent: sap.m.FlexJustifyContent.Center,
					items: [content],
				} );
			};

			const getTileContent = function () {
				const body = isGraphView ? getGraphViewContent() : content;

				return new sas.hc.m.VBox({
					justifyContent: sap.m.FlexJustifyContent.Start,
					items: [
						getTileHeader(),
						// SpacerFactory.verticalSpacer("0.2rem"), // remove this in order to move the donut a bit higher to let the tile have a more balanced view as a whole
						body,
					],
				} );
			};

			const getTile = function () {
				return new sas.hc.m.ObjectTile({
                    height: "9.5rem",  // "10rem"
					width: "13rem",    // "12rem" works but a bit tight for Biz Metrics, "14rem"
					bodyContent: getTileContent(),
					showContentHeaderSection: false,
					showFooterStatus: false,
					showObjectInspector: false,
					showToolbarSection: false,
				} );
			};

			const build = function () {
				const tile = getTile();
				if (isGraphView) {
					sas.ltjs.ltjsLoader.runWhenInitialized(graphViewSetUp);
				}
				setUp(tile);
				return tile;
			};

			return {
				build: build,
				setAdditionalInfo: setAdditionalInfo,
				setContent: setContent,
				setGraphViewSetUp: setGraphViewSetUp,
				setIcon: setIcon,
				setIsGraphView: setIsGraphView,
				setPopover: setPopover,
                setTitle: setTitle,
                setTooltip: setTooltip,
			};
		};

		const Decision = {
			APPROVE:  {code: 1, isApprove: true,      label: getText("rules.estimation.approve.decision.label.fmt.txt",  "1")},
			DECLINE:  {code: 4, isApprove: false,     label: getText("rules.estimation.decline.decision.label.fmt.txt",  "4")},
			EXCLUDE:  {code: 6, isApprove: true,      label: getText("rules.estimation.exclude.decision.label.fmt.txt",  "6")},
			IGNORE:   {code: 9, isApprove: true,      label: getText("rules.estimation.ignore.decision.label.fmt.txt",   "9")},
			PICKUP:   {code: 5, isApprove: false,     label: getText("rules.estimation.pickup.decision.label.fmt.txt",   "5")},
			REFERRAL: {code: 3, isApprove: false,     label: getText("rules.estimation.referral.decision.label.fmt.txt", "3")},
			UNKNOWN:  {code: 0, isApprove: undefined, label: getText("rules.estimation.unknown.decision.label.fmt.txt",  "0")},
			VERIFY:   {code: 2, isApprove: false,     label: getText("rules.estimation.verify.decision.label.fmt.txt",   "2")},
		};
		Object.freeze && Object.freeze(Decision);

		const DecisionsFactory = (function () {
            const getDecisionByCode = function (code) {
                const decisionCode = Number.isInteger(code) ? code : parseInteger(code);
                const allDecisions = Object.entries(Decision);
                const n = allDecisions.length;
                for (let i = 0; i < n; i++) {
                    const decision = allDecisions[i][1];
                    if (decision.code === decisionCode) {
                        return decision;
                    }
                }
                return null;
            };

			const sortDecisions = function (decisions) {
				decisions.sort(function (a, b) {
					return a.type.code > b.type.code;
				} );
				return decisions;
			};

            const newDecisions = function (decisions) { /* !Object<string, int> eg. {"0": 43, "1": 5, "2": 0, "3":5} */
                const answer = Object.entries(decisions).filter(function (each) {
                    return each[1] !== 0;
                } ).map(function (each) {
                    return {
                        type: getDecisionByCode(each[0]),
                        count: each[1],
                    };
                } );
            return sortDecisions(answer);
        };

			return {new: newDecisions};
        } )();

		const BizMetricsPopoverBuilder = function (decisionMetrics, estimationController) {
            const newAlertEntitiesSectionItems = function (sectionInfo) {
				const alertEntitiesSectionItems = Object.entries(sectionInfo.alertEntities).filter(function (each) {
                    return (each[1] !== 0);
                } ).map(function (each) {
					return {
                        label: estimationController.getAlertType(each[0]).fullDescription, // alert entity description & code
						value: formatter.formatNumber(each[1]),                            // alert entity value
						icon: Icon.TRIANGLE,
					};
                } );

				return [
					SpacerFactory.verticalSpacer("0.3rem"),
					newSectionTitleHBox(sectionInfo),
				].concat(PopoverLineRenderer.render(alertEntitiesSectionItems));
			};

			const newDecisionsDetailsSectionItems = function (section) {
				const answer = [
					SpacerFactory.verticalSpacer("0.8rem"),
					newSectionTitleHBox(section),
                ];
                let totalCount = 0;
				section.decisions.forEach(function (each) {
					if (each.count > 0) {
                        totalCount += each.count;
						const isApprove = each.type.isApprove;
						const icon = (isApprove === undefined)
							? Icon.QUESTION_MARK
							: isApprove ? Icon.CHECKMARK_GREEN : Icon.CROSS_RED;
						const sectionItem = {
							icon: icon,
							label: each.type.label,
							value: formatter.formatNumber(each.count),
						};
						answer.push(PopoverLineRenderer.render(sectionItem));
					}
				} );
				answer.push(PopoverLineRenderer.render({
					icon: Icon.SUM,
					label: getText("rules.estimation.total.label"),
					value: formatter.formatNumber(totalCount),
				} ) );
				return answer;
			};

			const build = function () {
                const entityDecisionCounts = decisionMetrics.entityDecisionCounts;
				const popoverItems = [
                    // The Alerts section is always shown.
					newPopoverSectionItems({
						isFirst: true,
						title: getText("rules.estimation.alerts.label"),
						content: {
							positive: {label: getText("rules.estimation.fraud.tp.label"), amount: formatter.formatNumber(entityDecisionCounts.truePositive)},
							negative: {label: getText("rules.estimation.false.alert.fp.label"), amount: formatter.formatNumber(entityDecisionCounts.falsePositive)},
							total: formatter.formatNumber(entityDecisionCounts.alerts),
						},
                    } )
                ];
                // The Alert Entities section is shown only if the number of alerts is > 0.
                if (decisionMetrics.entityDecisionCounts.alerts > 0) {
                    popoverItems.push(newAlertEntitiesSectionItems({
						title: getText("rules.estimation.alert.entities.title"),
						alertEntities: decisionMetrics.predictedEntityDecisionDistribution,
					} ) );
                }
                // The Decisions section is shown only if the decisions total is > 0.
                const decisions = DecisionsFactory.new(decisionMetrics.predictedTransactionDecisionDistribution);
                // Total the decision count and then determine if the Decisions section should be dispalyed.
                // Can't reply on decisionMetrics.transactionDecisionCounts.totalPopulation because for pre-v6 estimations
                //     decisionMetrics.transactionDecisionCounts.trueNegative is not defined, and as a result, totalPopulation
                //     is also NOT defined!
                const decisionsTotal = decisions.reduce(function (accumulator, each) {
                    return accumulator += each.count;
                }, 0 );
                if (decisionsTotal > 0) {
                    popoverItems.push(newDecisionsDetailsSectionItems({
						title: getText("rules.estimation.decisions.label"),
						decisions: decisions, // eg. {"0": 43, "1": 5, "2": 0, "3":5}
					} ) );
                }
                // The total Fired Transactions is always shown.
				popoverItems.push(newPopoverSectionItems({
					title: getText("rules.estimation.fired.transactions.title"),
					titleRightContent: formatter.formatNumber(decisionMetrics.firedCount),
				} ) );
				const answer = PopoverFactory.new({
					header: newPopoverHeader({leftContent: getText("rules.estimation.business.metrics.title")}),
					content: popoverItems.reduce(function (accumulator, currentValue) {
						return accumulator.concat(currentValue);
					}, [] ),
					contentWidth: Popover.CONTENT_WIDTH_PLUS,
					width: Popover.WIDTH_PLUS,
				} );
				return answer;
			};

			return {build: build};
		};

		const BizMetricsTileContentBuilder = function (decisionMetrics) {
            NullCheck.verify(decisionMetrics, NullCheck.LogError);

			const getBizMetricsTileSections = function () {
                const answer = [
					{id: "AlertsHBox",
						label: getText("rules.estimation.alerts.label"),
						value: formatter.formatNumber(decisionMetrics.entityDecisionCounts.alerts),
						icon: Icon.ALERT,
						topLevel: true,
					}, {id: "DecisionsHBox",
						label: getText("rules.estimation.decisions.label"),
						value: "",
						icon: Icon.DECISION,
						topLevel: true,
                    },
                ];
                const approveDecisions = decisionMetrics.transactionDecisionCounts.approveDecisions;
                answer.push({id: "DetailHBox",
                    label: getText("rules.estimation.approve.label"),
                    value: NumberAbbreviator.format(approveDecisions),
                    valueTooltip: formatter.formatNumber(approveDecisions),
                    icon: Icon.CHECKMARK_GREEN,
                    topLevel: false,
                } );
                const nonApproveDecisions = decisionMetrics.transactionDecisionCounts.nonApproveDecisions;
                answer.push({id: "DetailHBox",
						label: getText("rules.estimation.non-approve.label"),
                        value: NumberAbbreviator.format(nonApproveDecisions),
                        valueTooltip: formatter.formatNumber(nonApproveDecisions),
						icon: Icon.CROSS_RED,
						topLevel: false,
                } );
                answer.push({id: "DecisionsHBox",
						label: getText("rules.estimation.fired.label"),
						value: formatter.formatNumber(decisionMetrics.firedCount),
						icon: Icon.FLAG,
						topLevel: true,
				} );
				return answer;
			};

			const build = function () {
				const bizMetricsTileSections = getBizMetricsTileSections();
				const answer = new sas.hc.m.VBox({
					justifyContent: sap.m.FlexJustifyContent.Start,
					items: newTileSectionsItems(bizMetricsTileSections),
				} );
				answer.addEventDelegate({
					onAfterRendering: function (event) {
						const $srcControl = event.srcControl.$();
						$srcControl.find("[id*='AlertsHBox']").css("height", "22px");
						$srcControl.find("[id*='DecisionsHBox']").css("height", "22px");
						$srcControl.find("[id*='DetailHBox']").css("height", "18px");
					}
				} );
				return answer;
			};

			return {
				build: build,
			};
		};

        const BizMetricsTileBuilder = function (decisionMetrics, estimationController) {
            NullCheck.verify(decisionMetrics, NullCheck.LogError);

            const build = function () {
                const title = getText("rules.estimation.business.metrics.title");
                const bizMetricsTileContent = new BizMetricsTileContentBuilder(decisionMetrics).build();
                const bizMetricsPopover = new BizMetricsPopoverBuilder(decisionMetrics, estimationController).build();

                return new TileBuilder()
                    .setTitle(title)
                    .setIcon(Icon.METRICS)
                    .setContent(bizMetricsTileContent)
                    .setPopover(bizMetricsPopover)
                    .build();
            };

			return {
				build: build,
			};
        };

		const init = function (settings) {
			IdFactory.init(settings.idPrefix);
		};

        /*
         This is for the "All queues" row of the Alerts by Queue table.
         Eg.
            const qMetrics = [
                {"queueId":50200, "queueName":"D23_Q1", "alertTotal":1, "alerts":{"C":1} },
                {"queueId":50201, "queueName":"D23_Q2", "alertTotal":2, "alerts":{"A":1, "C":1}},
            ];
            console.log(new AllAlertsBuilder(qMetrics).build());
            =>
            {   queueId: 0,
                queueName: "All queues",
                alertTotal: 3,
                alerts: {C: 2, A: 1},
            }
         */
        const AllAlertsBuilder = function (allQueueMetrics) {
            const getAllAlerts = function () {
                const answer = {};
                allQueueMetrics.forEach(function (queueMetrics) {
                    Object.entries(queueMetrics.alerts).forEach(function (eachAlert) {
                        const alertCode = eachAlert[0];
                        const alertCount = eachAlert[1];
                        const alert = answer[alertCode];
                        if (alert === undefined) {
                            answer[alertCode] = 0;
                        }
                        answer[alertCode] += alertCount;
                    } );
                } );
                return answer;
            };

            const getAllAlertsTotal = function (allAlerts) {
                return Object.entries(allAlerts).reduce(function (accumulator, each) {
                      return accumulator += each[1];
                }, 0 );
            };

            const build = function () {
                const allAlerts = getAllAlerts();
                const alertCount = getAllAlertsTotal(allAlerts);
                return {
                    queueId: 0,
                    queueName: getText("rules.estimation.all.queues.label"),
                    alertTotal: alertCount,
                    alerts: allAlerts,
                };
            };

            return {
                build: build,
            };
        };

        // Build the "All rules" row for the Rules Decision Metrics table.
        const AllRulesBuilder = function (metrics) {
            const ALL_RULES_DISPLAY_ID = "99999999.9";
            var allRuleIds = [];
            metrics.ruleMetrics.forEach(function (each) {
            	allRuleIds[allRuleIds.length] = each.rule.id;
            }, this );

            const build = function () {
                const answer = {};
                answer.rule = {
                    id: "",
                    name: getText("rules.estimation.all.rules.label"),
                    state: "",
                    displayId: ALL_RULES_DISPLAY_ID,
                    isAllRules: true,
                    allRuleIds: allRuleIds
                };
                answer.entityDecisionCounts = metrics.decisionMetrics.entityDecisionCounts;
                answer.transactionDecisionCounts = metrics.decisionMetrics.transactionDecisionCounts;
                answer.transactionDecisionValues = metrics.decisionMetrics.transactionDecisionValues;
                return answer;
            };

            return {
                build: build,
            };
        };

        const newAlertEntitiesCountLink = function (estimationController, ruleTable) {
            const getProperties = function (ruleMetrics) {
                const rule = ruleMetrics.rule;
                const isHasAlertEntitiesCount = !!((rule.isAlertRule || rule.isAllRules)); // NOSONAR - ignore warning about useless parens.
                const alertEntitiesCount = isHasAlertEntitiesCount ? ruleMetrics.entityDecisionCounts.alertEntitiesCount : 0;
                const isToShow = (alertEntitiesCount > 0);

                return {
                    isHasAlertEntitiesCount: isHasAlertEntitiesCount,
                    alertEntitiesCount: alertEntitiesCount,
                    isToShow: isToShow,
                };
            };

            return new sas.hc.m.Link({
                text: {
                    path: "",
                    formatter: function (ruleMetrics) {
                        if (ruleMetrics) {
                            const properties = getProperties(ruleMetrics);
                            return (properties.isHasAlertEntitiesCount  && (properties.alertEntitiesCount >= 0))
                                    ? ruleMetrics.entityDecisionCounts.alertEntitiesCountAbbreviated
                                    : "";
                        } else {
                            return "";
                        }
                    },
                },                          /* jshint -W107 */
                href: "javascript:void(0)", /* jshint +W107 */
                tooltip: {
                    path: "",
                    formatter: function (ruleMetrics) {
                        if (ruleMetrics) {
                            const properties = getProperties(ruleMetrics);
                            return properties.isHasAlertEntitiesCount ? ruleMetrics.entityDecisionCounts.alertEntitiesCountLocalized : "";
                        } else {
                            return "";
                        }
                    },
                },
                press: function (event) {
                    const rowIndex = event.getSource().getParent().getParent().getIndex();
                    estimationController.showMatchedTransactions(ruleTable, rowIndex, true); // true means to show alerts, in addition to transactions
                },
                visible: {
                    path: "",
                    formatter: function (ruleMetrics) {
                        return (ruleMetrics) ? getProperties(ruleMetrics).isToShow : false;
                    },
                },
            } ).addEventDelegate({
                onAfterRendering: function (event) {
                    // Change the cursor to be the hand icon.  Not sure why we need to do this...
                    event.srcControl.getDomRef().style.setProperty("cursor", "pointer", "important");
                },
            } );
        };

        const newAlertEntitiesCountText = function () {
            const isToShowText = function (ruleMetrics) {
                if (ruleMetrics) {
                    const rule = ruleMetrics.rule;
                    const isHasAlertEntitiesCount = !!((rule.isAlertRule || rule.isAllRules));  // NOSONAR - ignore warning about useless parens.
                    return isHasAlertEntitiesCount && (ruleMetrics.entityDecisionCounts.alertEntitiesCount === 0);
                }
                else {
                    return false;
                }
            };

            return new sas.hc.m.Text({
				text: "0",
                visible: {
                    path: "",
                    formatter: function (ruleMetrics) {
                        return isToShowText(ruleMetrics);
                    },
                },
			} );
        };

        const newTrxnDecisionsCountLink = function (estimationController, ruleTable) {
            return new sas.hc.m.Link({
                text: "{transactionDecisionCounts/transactionDecisionsCountAbbreviated}", /* jshint -W107 */
                href: "javascript:void(0)",                                               /* jshint +W107 */
                tooltip: "{transactionDecisionCounts/transactionDecisionsCountLocalized}",
                press: function (event) {
                    const rowIndex = event.getSource().getParent().getParent().getIndex();
                    estimationController.showMatchedTransactions(ruleTable, rowIndex);
                },
                visible: {
                    path: "transactionDecisionCounts/predictedConditionPositive",
                    formatter: function (count) {
                        return (count > 0);
                    },
                },
            } );
        };

        const newTrxnDecisionsCountText = function () {
            return new sas.hc.m.Text({
				text: "0",
                visible: {
                    path: "transactionDecisionCounts/predictedConditionPositive",
                    formatter: function (count) {
                        return (count === 0);
                    },
                },
			} );
        };


	    return {
            AllAlertsBuilder: AllAlertsBuilder,
            AllRulesBuilder: AllRulesBuilder,
			BizMetricsTileBuilder: BizMetricsTileBuilder,
			Icon: Icon,
			Popover: Popover,
			PopoverFactory: PopoverFactory,
			TileBuilder: TileBuilder,

            init: init,
            newAlertEntitiesCountLink: newAlertEntitiesCountLink,
            newAlertEntitiesCountText: newAlertEntitiesCountText,
            newTrxnDecisionsCountLink: newTrxnDecisionsCountLink,
            newTrxnDecisionsCountText: newTrxnDecisionsCountText,
			newPopover: newPopover,
			newPopoverHeader: newPopoverHeader,
			newPopoverSectionItems: newPopoverSectionItems,
	    };
	} )(),
	true
);
